/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
draw.h

  Misc drawing routines.
*/

#ifndef _DRAW_H_
#define _DRAW_H_

#ifndef _DRAW_CPP_
#define _DRAW_CPP_ extern
#endif

#ifndef PI
#define PI 3.14159265359
#endif

#include <windows.h>
#include "point.h"

void transformSquarify(ptTransform2& tr, double x, double y);
void transformMakeFit(ptTransform2& tr, double x, double y, double sx, double sy);

Point2 unPolar(Point2 rad);

// Curve drawing
void drawCircle(HDC hDC, ptTransform2& tr, Point2 a, Point2 b);
void drawArc(HDC hDC, ptTransform2& tr, Point2 a, Point2 b, Point2 from, Point2 to);
void drawWedge(HDC hDC, ptTransform2& tr, double thStart, double thEnd, 
			   double r1, double r2, double resolution);	// note: angles in degrees
void drawBezier(HDC hDC, ptTransform2& tr, Point2 start, Point2 c1, Point2 c2, Point2 end);

// Line/polygon drawing
void drawLine(HDC hDC, ptTransform2& tr, Point2 a, Point2 b);
void drawRect(HDC hDC, ptTransform2& tr, Point2 a, Point2 b);
void drawQuad(HDC hDC, ptTransform2& tr, Point2 a, Point2 b, Point2 c, Point2 d);

// Pixel/dot drawing
void drawSetPixel(HDC hDC, ptTransform2& tr, Point2 pt, COLORREF clr);
void drawDot(HDC hDC, ptTransform2& tr, Point2 pt);
void drawFloodFill(HDC hDC, ptTransform2& tr, Point2 pt, COLORREF clr, int clrIsBorder = 1);

// Text drawing
HFONT drawMakeFont(HDC hDC, ptTransform2& tr, double Sz);
void drawText(HDC hDC, ptTransform2& tr, Point2 pt, char *text);

#endif
