/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
gauge.cpp

  Gauge controls.
*/

#include <stdio.h>
#include <windows.h>
#pragma hdrstop
#define _GAUGE_CPP_
#include "gauge.h"
#include "uavgs.h"
#include "packet.h"
#include "point.h"
#include "draw.h"
#include "sound.h"

#include "lcdchar.h"

COLORREF clrError = RGB(255, 0, 0);
HPEN hPenError;
HBRUSH hbrError;

COLORREF clrWarn = RGB(255, 255, 0);
HPEN hPenWarn;
HBRUSH hbrWarn;

HPEN hPenLCDon, hPenLCDoff;
HBRUSH hbrLCDon, hbrLCDoff, hbrLCDbkg;

void eraseGaugeBkg(HWND handle, HDC hDC, RECT *rc)
{
	HDC hMem;
	POINT pt;

	if (isTransparent)
	{
		FillRect(hDC, rc, hbrWndBkg);
		return;
	}

	// otherwise copy a chunk of the background bitmap
	pt.x = pt.y = 0;
	ClientToScreen(handle, &pt);
	ScreenToClient(hWnd1, &pt);

	hMem = CreateCompatibleDC(hDC);
	SelectObject(hMem, hbmpWndBkg);
	BitBlt(hDC, 0, 0, rc->right, rc->bottom, hMem, pt.x, pt.y, SRCCOPY);
	DeleteObject(hMem);
}

void drawAltimeter(HWND handle, HDC hDC)
{
	CVariable *var;
	double alt, altLo, altHi;
	ptTransform2 tr;
	RECT rc;
	int i;
	double th;
	Point2 pt;
	POINT pts[5];
	HFONT hf;
	char str[256];

	var = (CVariable*) GetProp(handle, "VAR");
	if (!var)
		return;

	alt = var->getNumber();
	altHi = fmod(alt / 10000.0f, 1.0f);		// ??x,yyy.0 <= altHi = X.YYY / 10
	altLo = fmod(alt / 1000.0f, 1.0f);		// ???,xyy.0 <= altLo = X.YY / 10

	GetClientRect(handle, &rc);
	transformSquarify(tr, (double) rc.right, (double) rc.bottom);

	// draw background
	SelectObject(hDC, hbrGaugeBkg);
	SelectObject(hDC, GetStockObject(NULL_PEN));
	drawCircle(hDC, tr, Point2(-1.0, -1.0), Point2(1.0, 1.0));

	// draw tick marks
	SelectObject(hDC, GetStockObject(WHITE_PEN));

	for (i = 0; i < 10; i++)
	{
		th = (double(i) / 10.0) * PI * 2.0;
		drawLine(hDC, tr, unPolar(Point2(0.8, th)), unPolar(Point2(0.55, th)));

		th = ((double(i) + 0.5) / 10.0) * PI * 2.0;
		drawLine(hDC, tr, unPolar(Point2(0.65, th)), unPolar(Point2(0.55, th)));
	}

	// draw text
	SetTextColor(hDC, RGB(255,255,255));
	SetBkMode(hDC, TRANSPARENT);
	hf = drawMakeFont(hDC, tr, 0.2);

	for (i = 0; i < 10; i++)
	{
		sprintf(str, "%i", i);

		th = (double(i) / 10.0) * PI * 2.0;		
		drawText(hDC, tr, unPolar(Point2(0.9, th)), str);
	}

	SelectObject(hDC, hfDefault);
	DeleteObject(hf);

	// draw needles
	SelectObject(hDC, GetStockObject(WHITE_PEN));
	SelectObject(hDC, GetStockObject(WHITE_BRUSH));
	th = altLo * PI * 2.0;
	pt = transform(tr, unPolar(Point2(0.75, th)));
	pts[0].x = int(pt.x); pts[0].y = int(pt.y);	// tip
	pt = transform(tr, unPolar(Point2(0.025, th - PI / 2.0)));
	pts[1].x = int(pt.x); pts[1].y = int(pt.y);	// side
	pt = transform(tr, unPolar(Point2(0.05, th + PI)));
	pts[2].x = int(pt.x); pts[2].y = int(pt.y);	// base
	pt = transform(tr, unPolar(Point2(0.025, th + PI / 2.0)));
	pts[3].x = int(pt.x); pts[3].y = int(pt.y);	// side
	pt = transform(tr, unPolar(Point2(0.75, th)));
	pts[4].x = int(pt.x); pts[4].y = int(pt.y);	// tip

	Polygon(hDC, pts, 5);

	th = altHi * PI * 2.0;
	pt = transform(tr, unPolar(Point2(0.55, th)));
	pts[0].x = int(pt.x); pts[0].y = int(pt.y);	// tip
	pt = transform(tr, unPolar(Point2(0.05, th - PI / 2.0)));
	pts[1].x = int(pt.x); pts[1].y = int(pt.y);	// side
	pt = transform(tr, unPolar(Point2(0.05, th + PI)));
	pts[2].x = int(pt.x); pts[2].y = int(pt.y);	// base
	pt = transform(tr, unPolar(Point2(0.05, th + PI / 2.0)));
	pts[3].x = int(pt.x); pts[3].y = int(pt.y);	// side
	pt = transform(tr, unPolar(Point2(0.55, th)));
	pts[4].x = int(pt.x); pts[4].y = int(pt.y);	// tip

	Polygon(hDC, pts, 5);
}
LRESULT CALLBACK wpAltimeter(HWND handle, UINT msg,
							 WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hDC, hMem;
	HBITMAP hBmp;
	RECT rc;

	switch (msg)
	{
	case WM_CLOSE:
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);
		break;
		
	case WM_SIZE:
		// need a new backbuffer
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);

		// redraw
		InvalidateRect(handle, 0, 1);
		UpdateWindow(handle);
		break;
	
	case WM_CREATE:
		break;

	case WM_PAINT:
		GetClientRect(handle, &rc);

		hDC = BeginPaint(handle, &ps);
		hMem = CreateCompatibleDC(hDC);

		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (!hBmp)
		{			
			hBmp = CreateCompatibleBitmap(hDC, rc.right, rc.bottom);
			SetProp(handle, "BACKBUF", (HANDLE) hBmp);
		}
		SelectObject(hMem, hBmp);

		eraseGaugeBkg(handle, hMem, &rc);
		drawAltimeter(handle, hMem);

		BitBlt(hDC, 0,0, rc.right, rc.bottom, hMem, 0,0, SRCCOPY);

		DeleteDC(hMem);
		EndPaint(handle, &ps);
		break;

	default:
		return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}

void drawCompass(HWND handle, HDC hDC)
{
	CVariable *var;
	double angle;
	ptTransform2 tr;
	RECT rc;
	int i;
	double th;
	Point2 pt;
	POINT pts[5];
	HFONT hf, hfL;

	var = (CVariable*) GetProp(handle, "VAR");
	if (!var)
		return;

	angle = var->getNumber() / 180.0 * PI;

	GetClientRect(handle, &rc);
	transformSquarify(tr, (double) rc.right, (double) rc.bottom);

	// draw background
	SelectObject(hDC, hbrGaugeBkg);
	SelectObject(hDC, GetStockObject(NULL_PEN));
	drawCircle(hDC, tr, Point2(-1.0, -1.0), Point2(1.0, 1.0));

	// draw tick marks
	SelectObject(hDC, GetStockObject(WHITE_PEN));

	for (i = 0; i < 8; i++)
	{
		th = (double(i) / 8.0) * PI * 2.0;
		drawLine(hDC, tr, unPolar(Point2(1.0, th)), unPolar(Point2(0.75, th)));

		th = ((double(i) + 0.5) / 8.0) * PI * 2.0;
		drawLine(hDC, tr, unPolar(Point2(1.0, th)), unPolar(Point2(0.9, th)));
	}

	// draw letters
	SetTextColor(hDC, RGB(255,255,255));
	SetBkMode(hDC, TRANSPARENT);
	
	hfL = drawMakeFont(hDC, tr, 0.25);
	pt = Point2(0.0, -0.6);	// N(orth)
	drawText(hDC, tr, pt, "N");
	hf = drawMakeFont(hDC, tr, 0.15);
	DeleteObject(hfL);

	pt = Point2(0.6, 0.0);	// E(ast)
	drawText(hDC, tr, pt, "E");
	pt = Point2(-0.6, 0.0);	// W(est)
	drawText(hDC, tr, pt, "W");
	pt = Point2(0.0, 0.6);	// S(outh)
	drawText(hDC, tr, pt, "S");

	SelectObject(hDC, hfDefault);
	DeleteObject(hf);

	// draw needle
	SelectObject(hDC, GetStockObject(WHITE_PEN));
	SelectObject(hDC, GetStockObject(WHITE_BRUSH));

	pt = transform(tr, unPolar(Point2(0.8, angle)));
	pts[0].x = int(pt.x); pts[0].y = int(pt.y);	// tip
	pt = transform(tr, unPolar(Point2(0.05, angle - PI / 2.0)));
	pts[1].x = int(pt.x); pts[1].y = int(pt.y);	// side
	pt = transform(tr, unPolar(Point2(0.15, angle + PI)));
	pts[2].x = int(pt.x); pts[2].y = int(pt.y);	// base
	pt = transform(tr, unPolar(Point2(0.05, angle + PI / 2.0)));
	pts[3].x = int(pt.x); pts[3].y = int(pt.y);	// side
	pt = transform(tr, unPolar(Point2(0.8, angle)));
	pts[4].x = int(pt.x); pts[4].y = int(pt.y);	// tip

	Polygon(hDC, pts, 5);
}
LRESULT CALLBACK wpCompass(HWND handle, UINT msg,
						   WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hDC, hMem;
	HBITMAP hBmp;
	RECT rc;

	switch (msg)
	{
	case WM_CLOSE:
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);
		break;
		
	case WM_SIZE:
		// need a new backbuffer
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);

		// redraw
		InvalidateRect(handle, 0, 1);
		UpdateWindow(handle);
		break;
	
	case WM_CREATE:
		break;

	case WM_PAINT:
		GetClientRect(handle, &rc);

		hDC = BeginPaint(handle, &ps);
		hMem = CreateCompatibleDC(hDC);

		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (!hBmp)
		{			
			hBmp = CreateCompatibleBitmap(hDC, rc.right, rc.bottom);
			SetProp(handle, "BACKBUF", (HANDLE) hBmp);
		}
		SelectObject(hMem, hBmp);

		eraseGaugeBkg(handle, hMem, &rc);
		drawCompass(handle, hMem);
		BitBlt(hDC, 0,0, rc.right, rc.bottom, hMem, 0,0, SRCCOPY);

		DeleteDC(hMem);
		EndPaint(handle, &ps);
		break;

	default:
		return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}

void drawRoundGauge(HWND handle, HDC hDC)
{
	CVariable *var;
	CWndFrameObj *wfo;
	double angle;
	ptTransform2 tr;
	RECT rc;
	double th, d, val;
	Point2 pt;
	POINT pts[5];
	bool err, warn;

	var = (CVariable*) GetProp(handle, "VAR");
	if (!var)
		return;
	wfo = (CWndFrameObj*) GetProp(handle, "WFO");
	if (!wfo)
		return;

	err = warn = 0;

	// Convert number to an angle
	val = var->getNumber();
	angle = (val - wfo->roundGauge.minValue) /
		(wfo->roundGauge.maxValue - wfo->roundGauge.minValue);
	if (angle < 0.0) 
		angle = 0.0;
	if (angle > 1.0) 
		angle = 1.0;

	angle = ( (angle * (wfo->roundGauge.maxAngle - wfo->roundGauge.minAngle)) 
		+ wfo->roundGauge.minAngle ) * PI / 180.0;

	GetClientRect(handle, &rc);
	transformSquarify(tr, (double) rc.right, (double) rc.bottom);

	// draw background
	SelectObject(hDC, hbrGaugeBkg);
	SelectObject(hDC, GetStockObject(NULL_PEN));
	drawCircle(hDC, tr, Point2(-1.0, -1.0), Point2(1.0, 1.0));

	// draw warning area arcs
	SelectObject(hDC, hPenWarn);
	SelectObject(hDC, hbrWarn);
	if (wfo->roundGauge.doWarnLo)
	{
		if (val <= wfo->roundGauge.warnLo)
		{
			warn = 1;
			beepProblem |= BEEP_WARN;
		}

		th = wfo->roundGauge.warnLo;
		th = (th - wfo->roundGauge.minValue) /
			(wfo->roundGauge.maxValue - wfo->roundGauge.minValue);
		th = ( (th * (wfo->roundGauge.maxAngle - wfo->roundGauge.minAngle)) 
			+ wfo->roundGauge.minAngle );

		if (wfo->roundGauge.doErrorLo)
		{
			d = wfo->roundGauge.errorLo;
			d = (d - wfo->roundGauge.minValue) /
				(wfo->roundGauge.maxValue - wfo->roundGauge.minValue);
			d = ( (d * (wfo->roundGauge.maxAngle - wfo->roundGauge.minAngle)) 
				+ wfo->roundGauge.minAngle );
		} else
			d = wfo->roundGauge.minAngle;

		drawWedge(hDC, tr, d, th, 0.8, 1.0, 5.0);
	}
	if (wfo->roundGauge.doWarnHi)
	{
		if (val >= wfo->roundGauge.warnHi)
		{
			warn = 1;
			beepProblem |= BEEP_WARN;
		}

		th = wfo->roundGauge.warnHi;
		th = (th - wfo->roundGauge.minValue) /
			(wfo->roundGauge.maxValue - wfo->roundGauge.minValue);
		th = ( (th * (wfo->roundGauge.maxAngle - wfo->roundGauge.minAngle)) 
			+ wfo->roundGauge.minAngle );

		if (wfo->roundGauge.doErrorHi)
		{
			d = wfo->roundGauge.errorHi;
			d = (d - wfo->roundGauge.minValue) /
				(wfo->roundGauge.maxValue - wfo->roundGauge.minValue);
			d = ( (d * (wfo->roundGauge.maxAngle - wfo->roundGauge.minAngle)) 
				+ wfo->roundGauge.minAngle );
		} else
			d = wfo->roundGauge.maxAngle;

		drawWedge(hDC, tr, th, d, 0.8, 1.0, 5.0);
	}

	SelectObject(hDC, hPenError);
	SelectObject(hDC, hbrError);
	if (wfo->roundGauge.doErrorLo)
	{
		if (val <= wfo->roundGauge.errorLo)
		{
			err = 1;
			beepProblem |= BEEP_ERROR;
		}

		th = wfo->roundGauge.errorLo;
		th = (th - wfo->roundGauge.minValue) /
			(wfo->roundGauge.maxValue - wfo->roundGauge.minValue);
		th = ( (th * (wfo->roundGauge.maxAngle - wfo->roundGauge.minAngle)) 
			+ wfo->roundGauge.minAngle );

		drawWedge(hDC, tr, wfo->roundGauge.minAngle, th, 0.8, 1.0, 5.0);
	}
	if (wfo->roundGauge.doErrorHi)
	{
		if (val >= wfo->roundGauge.errorHi)
		{
			err = 1;
			beepProblem |= BEEP_ERROR;
		}

		th = wfo->roundGauge.errorHi;
		th = (th - wfo->roundGauge.minValue) /
			(wfo->roundGauge.maxValue - wfo->roundGauge.minValue);
		th = ( (th * (wfo->roundGauge.maxAngle - wfo->roundGauge.minAngle)) 
			+ wfo->roundGauge.minAngle );

		drawWedge(hDC, tr, th, wfo->roundGauge.maxAngle, 0.8, 1.0, 5.0);
	}

	// draw tick marks
	SelectObject(hDC, GetStockObject(WHITE_PEN));

	if (wfo->roundGauge.doTicks)
	{
		for (d = wfo->roundGauge.minValue; d < wfo->roundGauge.maxValue;
		     d += wfo->roundGauge.tickSpace)
		{
			th = (d - wfo->roundGauge.minValue) /
				(wfo->roundGauge.maxValue - wfo->roundGauge.minValue);

			th = ( (th * (wfo->roundGauge.maxAngle - wfo->roundGauge.minAngle)) 
				+ wfo->roundGauge.minAngle ) * PI / 180.0;

			drawLine(hDC, tr, unPolar(Point2(1.0, th)), unPolar(Point2(0.75, th)));
		}
		// the last tick may not quite make it to the maximum value (spacing
		// doesn't divide evenly into gauge range)
		// add one at the maximum value
		th = wfo->roundGauge.maxAngle * PI / 180.0;
		drawLine(hDC, tr, unPolar(Point2(1.0, th)), unPolar(Point2(0.75, th)));
	}
	if (wfo->roundGauge.doTicks2)
	{
		for (d = wfo->roundGauge.minValue; d < wfo->roundGauge.maxValue;
		     d += wfo->roundGauge.tickSpace2)
		{
			th = (d - wfo->roundGauge.minValue) /
				(wfo->roundGauge.maxValue - wfo->roundGauge.minValue);

			th = ( (th * (wfo->roundGauge.maxAngle - wfo->roundGauge.minAngle)) 
				+ wfo->roundGauge.minAngle ) * PI / 180.0;

			drawLine(hDC, tr, unPolar(Point2(1.0, th)), unPolar(Point2(0.9, th)));
		}
		// the last tick may not quite make it to the maximum value (spacing
		// doesn't divide evenly into gauge range)
		// add one at the maximum value
		th = wfo->roundGauge.maxAngle * PI / 180.0;
		drawLine(hDC, tr, unPolar(Point2(1.0, th)), unPolar(Point2(0.9, th)));
	}

	// draw warning/error light
	if (gaugeFlash && (err || warn))
	{
		if (err)
		{
			SelectObject(hDC, hbrError);
			SelectObject(hDC, hPenError);
		} else {
			SelectObject(hDC, hbrWarn);
			SelectObject(hDC, hPenWarn);
		}

		drawCircle(hDC, tr, Point2(-0.1, 0.15), Point2(0.1, 0.35));
	}

	// draw needle
	SelectObject(hDC, GetStockObject(NULL_PEN));
	SelectObject(hDC, GetStockObject(GRAY_BRUSH));
	drawCircle(hDC, tr, Point2(-0.1, -0.1), Point2(0.1,0.1));

	SelectObject(hDC, GetStockObject(WHITE_PEN));
	SelectObject(hDC, GetStockObject(WHITE_BRUSH));

	pt = transform(tr, unPolar(Point2(0.8, angle)));
	pts[0].x = int(pt.x); pts[0].y = int(pt.y);	// tip
	pt = transform(tr, unPolar(Point2(0.05, angle - PI / 2.0)));
	pts[1].x = int(pt.x); pts[1].y = int(pt.y);	// side
	pt = transform(tr, unPolar(Point2(0.05, angle + PI)));
	pts[2].x = int(pt.x); pts[2].y = int(pt.y);	// base
	pt = transform(tr, unPolar(Point2(0.05, angle + PI / 2.0)));
	pts[3].x = int(pt.x); pts[3].y = int(pt.y);	// side
	pt = transform(tr, unPolar(Point2(0.8, angle)));
	pts[4].x = int(pt.x); pts[4].y = int(pt.y);	// tip

	Polygon(hDC, pts, 5);
}
LRESULT CALLBACK wpRoundGauge(HWND handle, UINT msg,
							  WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hDC, hMem;
	HBITMAP hBmp;
	RECT rc;

	switch (msg)
	{
	case WM_CLOSE:
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);
		break;
		
	case WM_SIZE:
		// need a new backbuffer
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);

		// redraw
		InvalidateRect(handle, 0, 1);
		UpdateWindow(handle);
		break;
	
	case WM_CREATE:
		break;

	case WM_PAINT:
		GetClientRect(handle, &rc);

		hDC = BeginPaint(handle, &ps);
		hMem = CreateCompatibleDC(hDC);

		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (!hBmp)
		{			
			hBmp = CreateCompatibleBitmap(hDC, rc.right, rc.bottom);
			SetProp(handle, "BACKBUF", (HANDLE) hBmp);
		}
		SelectObject(hMem, hBmp);

		eraseGaugeBkg(handle, hMem, &rc);
		drawRoundGauge(handle, hMem);
		BitBlt(hDC, 0,0, rc.right, rc.bottom, hMem, 0,0, SRCCOPY);

		DeleteDC(hMem);
		EndPaint(handle, &ps);
		break;

	default:
		return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}

void drawLCDdigit(HDC hDC, ptTransform2& tr, Point2 pt, int mask)
{
	// decpoint
	if (mask & 128)
	{
		SelectObject(hDC, hbrLCDon);
		SelectObject(hDC, hPenLCDon);		
	} else {
		SelectObject(hDC, hbrLCDoff);
		SelectObject(hDC, hPenLCDoff);
	}
	drawDot(hDC, tr, pt + Point2(1.0, 1.0));

	// top
	if (mask & 64)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.3, 0.0), pt + Point2(0.7, 0.0));

	// left
	if (mask & 32)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.2, 0.1), pt + Point2(0.2, 0.4));

	// right
	if (mask & 16)		
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.8, 0.1), pt + Point2(0.8, 0.4));

	// middle
	if (mask & 8)		
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.3, 0.5), pt + Point2(0.7, 0.5));

	// left
	if (mask & 4)		
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.2, 0.6), pt + Point2(0.2, 0.9));

	// right
	if (mask & 2)		
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.8, 0.6), pt + Point2(0.8, 0.9));

	// bottom
	if (mask & 1)		
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.3, 1.0), pt + Point2(0.7, 1.0));
}
void drawLCD(HWND handle, HDC hDC)
{
	CVariable *var;
	CWndFrameObj *wfo;
	double value, digit;
	int digNum, d;
	ptTransform2 tr;
	Point2 pt;
	RECT rc;
	int decpoint;

	var = (CVariable*) GetProp(handle, "VAR");
	if (!var)
		return;
	wfo = (CWndFrameObj*) GetProp(handle, "WFO");
	if (!wfo)
		return;

	value = var->getNumber();

	GetClientRect(handle, &rc);
	transformMakeFit(tr, (double) rc.right, (double) rc.bottom, wfo->lcd.nDigits * 1.1, 1.5);

	// erase area around the gauge
	FillRect(hDC, &rc, hbrGaugeBkg);

	// draw display
	for (digNum = 0, digit = wfo->lcd.msDigit; digNum < wfo->lcd.nDigits; 
	     digNum++, digit /= 10.0)
	{
		pt = Point2(double(digNum) * 1.1, 0.25);

		// draw digit background
		SelectObject(hDC, hbrLCDbkg);
		SelectObject(hDC, GetStockObject(NULL_PEN));
		drawRect(hDC, tr, pt - Point2(0.0, 0.1), pt + Point2(1.0, 1.1));

		// draw digit segments
		if (value >= 0.0)
			d = int( floor(fmod(value / digit, 10.0)) );
		else
			d++;

		if ((digit > 0.9) && (digit < 1.1))
			decpoint = 128;
		else
			decpoint = 0;

		switch (d)
		{
		case 0:
			drawLCDdigit(hDC, tr, pt, decpoint | 64 | 32 | 16 | 0 | 4 | 2 | 1);
			break;
		case 1:
			drawLCDdigit(hDC, tr, pt, decpoint |  0 |  0 | 16 | 0 | 0 | 2 | 0);
			break;
		case 2:
			drawLCDdigit(hDC, tr, pt, decpoint | 64 |  0 | 16 | 8 | 4 | 0 | 1);
			break;
		case 3:
			drawLCDdigit(hDC, tr, pt, decpoint | 64 |  0 | 16 | 8 | 0 | 2 | 1);
			break;
		case 4:
			drawLCDdigit(hDC, tr, pt, decpoint |  0 | 32 | 16 | 8 | 0 | 2 | 0);
			break;
		case 5:
			drawLCDdigit(hDC, tr, pt, decpoint | 64 | 32 |  0 | 8 | 0 | 2 | 1);
			break;
		case 6:
			drawLCDdigit(hDC, tr, pt, decpoint | 64 | 32 |  0 | 8 | 4 | 2 | 1);
			break;
		case 7:
			drawLCDdigit(hDC, tr, pt, decpoint | 64 |  0 | 16 | 0 | 0 | 2 | 0);
			break;
		case 8:
			drawLCDdigit(hDC, tr, pt, decpoint | 64 | 32 | 16 | 8 | 4 | 2 | 1);
			break;
		case 9:
			drawLCDdigit(hDC, tr, pt, decpoint | 64 | 32 | 16 | 8 | 0 | 2 | 1);
			break;
		default:
			drawLCDdigit(hDC, tr, pt, decpoint |  0 |  0 |  0 | 8 | 0 | 0 | 0);
			break;
		}
	}
}
LRESULT CALLBACK wpLCD(HWND handle, UINT msg,
					   WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hDC, hMem;
	HBITMAP hBmp;
	RECT rc;

	switch (msg)
	{
	case WM_CLOSE:
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);
		break;
		
	case WM_SIZE:
		// need a new backbuffer
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);

		// redraw
		InvalidateRect(handle, 0, 1);
		UpdateWindow(handle);
		break;
	
	case WM_CREATE:
		break;

	case WM_PAINT:
		GetClientRect(handle, &rc);

		hDC = BeginPaint(handle, &ps);
		hMem = CreateCompatibleDC(hDC);

		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (!hBmp)
		{			
			hBmp = CreateCompatibleBitmap(hDC, rc.right, rc.bottom);
			SetProp(handle, "BACKBUF", (HANDLE) hBmp);
		}
		SelectObject(hMem, hBmp);

		eraseGaugeBkg(handle, hMem, &rc);
		drawLCD(handle, hMem);
		BitBlt(hDC, 0,0, rc.right, rc.bottom, hMem, 0,0, SRCCOPY);

		DeleteDC(hMem);
		EndPaint(handle, &ps);
		break;

	default:
		return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}

/*
 mask1: (vert/horz) binary ABCDEFGHI
 mask2: (diagonals) binary XYZW

  ----A----
  |   |   |
  B X C Y D
  |   |   |
  ----E----
  |   |   |
  F Z G W H
  |   |   |
  ----I----
*/
void drawLCDletter(HDC hDC, ptTransform2& tr, Point2 pt, int mask1, int mask2)
{
	// top horizontal
	if (mask1 & 256)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.3, 0.0), pt + Point2(0.7, 0.0));

	// left
	if (mask1 & 128)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.2, 0.1), pt + Point2(0.2, 0.4));

	// center
	if (mask1 & 64)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.5, 0.1), pt + Point2(0.5, 0.4));

	// right
	if (mask1 & 32)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.8, 0.1), pt + Point2(0.8, 0.4));

	// middle horizontal
	if (mask1 & 16)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.3, 0.5), pt + Point2(0.7, 0.5));

	// left
	if (mask1 & 8)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.2, 0.6), pt + Point2(0.2, 0.9));

	// center
	if (mask1 & 4)		
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.5, 0.6), pt + Point2(0.5, 0.9));

	// right
	if (mask1 & 2)		
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.8, 0.6), pt + Point2(0.8, 0.9));

	// bottom horizontal
	if (mask1 & 1)		
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.3, 1.0), pt + Point2(0.7, 1.0));

	/*** diagonals ***/
	if (mask2 & 8)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.4, 0.4), pt + Point2(0.3, 0.1));

	if (mask2 & 4)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.6, 0.4), pt + Point2(0.7, 0.1));

	if (mask2 & 2)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.4, 0.6), pt + Point2(0.3, 0.9));

	if (mask2 & 1)
		SelectObject(hDC, hPenLCDon);
	else
		SelectObject(hDC, hPenLCDoff);
	drawLine(hDC, tr, pt + Point2(0.6, 0.6), pt + Point2(0.7, 0.9));
}
void drawLCDText(HWND handle, HDC hDC)
{
	CVariable *var;
	CWndFrameObj *wfo;
	char *str;
	ptTransform2 tr;
	Point2 pt;
	RECT rc;
	int i, ch, found;

	var = (CVariable*) GetProp(handle, "VAR");
	if (!var)
		return;
	wfo = (CWndFrameObj*) GetProp(handle, "WFO");
	if (!wfo)
		return;

	str = var->getString();

	GetClientRect(handle, &rc);
	transformMakeFit(tr, (double) rc.right, (double) rc.bottom, wfo->lcdtext.size * 1.1, 1.5);

	// erase area around the gauge
	FillRect(hDC, &rc, hbrGaugeBkg);

	// draw display
	for (i = 0; i < wfo->lcdtext.size; i++)
	{
		pt = Point2(double(i) * 1.1, 0.25);

		// draw letter background
		SelectObject(hDC, hbrLCDbkg);
		SelectObject(hDC, GetStockObject(NULL_PEN));
		drawRect(hDC, tr, pt - Point2(0.0, 0.1), pt + Point2(1.0, 1.1));

		// draw letter segments
		if (i >= (int) strlen(str))
			drawLCDletter(hDC, tr, pt, 0, 0);
		else {
			for (ch = found = 0; !found && lcdChar[ch].chr; ch++)
			{
				if (lcdChar[ch].chr == (char) toupper(str[i]))
				{
					found = 1;
					drawLCDletter(hDC, tr, pt, lcdChar[ch].mask1, lcdChar[ch].mask2);
				}
			}
			
			// last one is default
			if (!found)
				drawLCDletter(hDC, tr, pt, lcdChar[ch].mask1, lcdChar[ch].mask2);
		}
	}
}
LRESULT CALLBACK wpLCDText(HWND handle, UINT msg,
						   WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hDC, hMem;
	HBITMAP hBmp;
	RECT rc;

	switch (msg)
	{
	case WM_CLOSE:
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);
		break;
		
	case WM_SIZE:
		// need a new backbuffer
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);

		// redraw
		InvalidateRect(handle, 0, 1);
		UpdateWindow(handle);
		break;
	
	case WM_CREATE:
		break;

	case WM_PAINT:
		GetClientRect(handle, &rc);

		hDC = BeginPaint(handle, &ps);
		hMem = CreateCompatibleDC(hDC);

		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (!hBmp)
		{			
			hBmp = CreateCompatibleBitmap(hDC, rc.right, rc.bottom);
			SetProp(handle, "BACKBUF", (HANDLE) hBmp);
		}
		SelectObject(hMem, hBmp);

		eraseGaugeBkg(handle, hMem, &rc);
		drawLCDText(handle, hMem);
		BitBlt(hDC, 0,0, rc.right, rc.bottom, hMem, 0,0, SRCCOPY);

		DeleteDC(hMem);
		EndPaint(handle, &ps);
		break;

	default:
		return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}

int regClassesGauge()
{
	WNDCLASS wc;

	// Make pens, brushes
	hPenError = CreatePen(PS_SOLID, 0, clrError);
	hbrError = CreateSolidBrush(clrError);

	hPenWarn = CreatePen(PS_SOLID, 0, clrWarn);
	hbrWarn = CreateSolidBrush(clrWarn);

	hPenLCDon  = CreatePen(PS_SOLID, 0, RGB( 16, 255,  64));
	hbrLCDon   = CreateSolidBrush(RGB( 16, 255,  64));
	
	hPenLCDoff = CreatePen(PS_SOLID, 0, RGB( 16,  64,  64));
	hbrLCDoff  = CreateSolidBrush(RGB( 16,  64,  64));
	
	hbrLCDbkg  = CreateSolidBrush(RGB( 32,  32,  32));

	// Altimeter
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpAltimeter;
	wc.hInstance	 = hInstance;
	wc.hCursor		 = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
	wc.lpszClassName = "UAVGSalt";
	if (!RegisterClass(&wc))
		return 0;

	// Compass
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpCompass;
	wc.hInstance	 = hInstance;
	wc.hCursor		 = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
	wc.lpszClassName = "UAVGScompass";
	if (!RegisterClass(&wc))
		return 0;

	// Round gauge
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpRoundGauge;
	wc.hInstance	 = hInstance;
	wc.hCursor		 = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
	wc.lpszClassName = "UAVGSroundg";
	if (!RegisterClass(&wc))
		return 0;

	// LCD
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpLCD;
	wc.hInstance	 = hInstance;
	wc.hCursor		 = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
	wc.lpszClassName = "UAVGSlcd";
	if (!RegisterClass(&wc))
		return 0;

	// LCD Text
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpLCDText;
	wc.hInstance	 = hInstance;
	wc.hCursor		 = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
	wc.lpszClassName = "UAVGSlcdtxt";
	if (!RegisterClass(&wc))
		return 0;

	return 1;
}

