/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
 lcdchar.h

 Text LCD character table
 */

struct sLCDChar {
	char chr;
	int mask1, mask2;
};

#define LCD_MASK1(A,B,C,D,E,F,G,H,I) \
	( ((A) ? 256:0) | ((B) ? 128:0) | ((C) ? 64:0) | ((D) ? 32:0) | ((E) ? 16:0) | \
	  ((F) ? 8:0) | ((G) ? 4:0) | ((H) ? 2:0) | ((I) ? 1:0) )
#define LCD_MASK2(X,Y,Z,W) \
	( ((X) ? 8:0) | ((Y) ? 4:0) | ((Z) ? 2:0) | ((W) ? 1:0) )

/*
 mask1: (vert/horz) binary ABCDEFGHI
 mask2: (diagonals) binary XYZW

  ----A----
  |\  |  /|
  B X C Y D
  |  \|/  |
  ----E----
  |  /|\  |
  F Z G W H
  |/  |  \|
  ----I----
*/

sLCDChar lcdChar[] = {
	{'A', LCD_MASK1(1, 1,0,1, 1, 1,0,1, 0), LCD_MASK2(0, 0, 0, 0)},
	{'B', LCD_MASK1(1, 1,0,0, 1, 1,0,1, 1), LCD_MASK2(0, 1, 0, 0)},
	{'C', LCD_MASK1(1, 1,0,0, 0, 1,0,0, 1), LCD_MASK2(0, 0, 0, 0)},
	{'D', LCD_MASK1(1, 0,1,1, 0, 0,1,1, 1), LCD_MASK2(0, 0, 0, 0)},
	{'E', LCD_MASK1(1, 1,0,0, 1, 1,0,0, 1), LCD_MASK2(0, 0, 0, 0)},
	{'F', LCD_MASK1(1, 1,0,0, 1, 1,0,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{'G', LCD_MASK1(1, 1,0,0, 0, 1,0,0, 1), LCD_MASK2(0, 0, 0, 1)},
	{'H', LCD_MASK1(0, 1,0,1, 1, 1,0,1, 0), LCD_MASK2(0, 0, 0, 0)},
	{'I', LCD_MASK1(1, 0,1,0, 0, 0,1,0, 1), LCD_MASK2(0, 0, 0, 0)},
	{'J', LCD_MASK1(0, 0,0,1, 0, 1,0,1, 1), LCD_MASK2(0, 0, 0, 0)},
	{'K', LCD_MASK1(0, 0,1,0, 0, 0,1,0, 0), LCD_MASK2(0, 1, 0, 1)},
	{'L', LCD_MASK1(0, 1,0,0, 0, 1,0,0, 1), LCD_MASK2(0, 0, 0, 0)},
	{'M', LCD_MASK1(0, 1,0,1, 0, 1,1,1, 0), LCD_MASK2(1, 1, 0, 0)},
	{'N', LCD_MASK1(0, 1,0,1, 0, 1,0,1, 0), LCD_MASK2(1, 0, 0, 1)},
	{'O', LCD_MASK1(1, 1,0,1, 0, 1,0,1, 1), LCD_MASK2(0, 0, 0, 0)},
	{'P', LCD_MASK1(1, 1,0,1, 1, 1,0,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{'Q', LCD_MASK1(1, 1,0,1, 0, 1,0,1, 1), LCD_MASK2(0, 0, 0, 1)},
	{'R', LCD_MASK1(1, 1,0,1, 1, 1,1,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{'S', LCD_MASK1(1, 0,0,0, 0, 0,0,0, 1), LCD_MASK2(1, 0, 0, 1)},
	{'T', LCD_MASK1(1, 0,1,0, 0, 0,1,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{'U', LCD_MASK1(0, 1,0,1, 0, 1,0,1, 1), LCD_MASK2(0, 0, 0, 0)},
	{'V', LCD_MASK1(0, 1,0,0, 0, 1,0,0, 0), LCD_MASK2(0, 1, 1, 0)},
	{'W', LCD_MASK1(0, 1,0,1, 0, 1,0,1, 0), LCD_MASK2(0, 0, 1, 1)},
	{'X', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(1, 1, 1, 1)},
	{'Y', LCD_MASK1(0, 0,0,0, 0, 0,1,0, 0), LCD_MASK2(1, 1, 0, 0)},
	{'Z', LCD_MASK1(1, 0,0,0, 0, 0,0,0, 1), LCD_MASK2(0, 1, 1, 0)},

	{'0', LCD_MASK1(1, 1,0,1, 0, 1,0,1, 1), LCD_MASK2(0, 1, 1, 0)},
	{'1', LCD_MASK1(0, 0,1,0, 0, 0,1,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{'2', LCD_MASK1(1, 0,0,1, 1, 1,0,0, 1), LCD_MASK2(0, 0, 0, 0)},
	{'3', LCD_MASK1(1, 0,0,1, 1, 0,0,1, 1), LCD_MASK2(0, 0, 0, 0)},
	{'4', LCD_MASK1(0, 1,0,1, 1, 0,0,1, 0), LCD_MASK2(0, 0, 0, 0)},
	{'5', LCD_MASK1(1, 1,0,0, 1, 0,0,1, 1), LCD_MASK2(0, 0, 0, 0)},
	{'6', LCD_MASK1(1, 1,0,0, 1, 1,0,1, 1), LCD_MASK2(0, 0, 0, 0)},
	{'7', LCD_MASK1(1, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(0, 1, 1, 0)},
	{'8', LCD_MASK1(1, 1,0,1, 1, 1,0,1, 1), LCD_MASK2(0, 0, 0, 0)},
	{'9', LCD_MASK1(1, 1,0,1, 1, 0,0,1, 1), LCD_MASK2(0, 0, 0, 0)},

	{' ', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{'-', LCD_MASK1(0, 0,0,0, 1, 0,0,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{'_', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 1), LCD_MASK2(0, 0, 0, 0)},
	{'(', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(0, 1, 0, 1)},
	{')', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(1, 0, 1, 0)},
	{'<', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(0, 1, 0, 1)},
	{'>', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(1, 0, 1, 0)},
	{'{', LCD_MASK1(0, 0,0,0, 1, 0,0,0, 0), LCD_MASK2(0, 1, 0, 1)},
	{'}', LCD_MASK1(0, 0,0,0, 1, 0,0,0, 0), LCD_MASK2(1, 0, 1, 0)},
	{'?', LCD_MASK1(1, 0,0,1, 1, 1,0,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{'/', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(0, 1, 1, 0)},
	{'\\', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(1, 0, 0, 1)},
	{'+', LCD_MASK1(0, 0,1,0, 1, 0,1,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{'*', LCD_MASK1(0, 0,1,0, 1, 0,1,0, 0), LCD_MASK2(1, 1, 1, 1)},
	{'`', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(1, 0, 0, 0)},
	{'\'', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(0, 1, 0, 0)},
	{'\"', LCD_MASK1(0, 0,1,1, 0, 0,0,0, 0), LCD_MASK2(0, 0, 0, 0)},
	{',', LCD_MASK1(0, 0,0,0, 0, 0,0,0, 0), LCD_MASK2(0, 0, 1, 0)},

	// default is all segments on
	{0,   LCD_MASK1(1, 1,1,1, 1, 1,1,1, 1), LCD_MASK2(1, 1, 1, 1)}
};
