/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
linkedlist.h

  Generic linked list.
*/

#ifndef _LNKLIST_H_
#define _LNKLIST_H_

#ifndef _LNKLIST_CPP_
#define _LNKLIST_CPP_ extern
#endif

template <class type> class CLList
{
protected:
	class llnode {
	public:
		type data;
		llnode *next;

		llnode()
		{
		}
		~llnode()
		{
			delete data;
		}
	};
	llnode *head;
	llnode *ptr;		// The current node

public:
	CLList() { init(); }
	~CLList() { destroy(); }

	void init()				// Initialize the list
	{
		head = ptr = 0;
	}

	void home()				// Return to the beginning of the list
	{
		ptr = head;
	}
	
	void destroy()			// Destroy the whole list
	{
		llnode *p;
		
		p = head;
		while (p)
		{
			head = head->next;
			delete p;
			p = head;
		}
		ptr = head = 0;
	}
	
	void add(type obj)		// Add a new node just after the current node
	{
		llnode *n;
		
		n = new llnode;
		memcpy(&(n->data), &obj, sizeof(obj));
		
		if (!ptr) ptr = head;

		if (!ptr)
		{
			ptr = head = n;
			n->next = 0;
		} else {
			n->next = ptr->next;
			ptr->next = n;
		}

		ptr = n;
	}
	void append(type obj)		// Add a new node at the end of the list
	{
		ptr = head;
		while (ptr ? ptr->next : 0) ptr = ptr->next;

		add(obj);
	}
	void eliminate(type obj)	// Search for & destroy all of these
	{		
		llnode *n;

		while (head)
		{
			if (head->data == obj)
			{
				n = head->next;
				delete head;
				head = n;
			} else break;
		}
		if (!head) return;

		ptr = head;
		while (ptr ? ptr->next : 0)
		{
			n = ptr->next;
			if (n)
				if (n->data == obj)
				{				
					ptr->next = n->next;
					delete n;
					n = ptr->next;
				}
			ptr = n;
		}
	}
	void remove()			// Delete the selected node
	{
		llnode *n, *sel;

		if (!head) return;

		if (ptr == head)
		{
			// Delete the head, shift everything up
			// The next node is selected.
			n = head->next;
			delete head;
			ptr = head = n;
		} else {
			sel = ptr;
			ptr = head;
			while (ptr ? ptr->next : 0)
			{
				n = ptr->next;
				if (n == sel)
				{
					// Aha! Found it!
					// The previous node is selected.
					ptr->next = n->next;
					delete n;
					return;
				}
				ptr = n;
			}
		}		
	}

	type* pdata()			// Returns the current node's data
	{
		return &ptr->data;
	}
	type& data()			// Returns the current node's data
	{
		return ptr->data;
	}
	
	CLList<type>& operator ++ ()
	{
		if (ptr)
			ptr = ptr->next;
			
		return *this;
	}

	bool end()			// Returns 0 if there is a 'next' node
	{
		return ptr ? (ptr->next ? 0 : 1) : 1;
	}
	bool pastend()		// Returns 0 if 'ptr' is valid
	{
		return ptr ? 0 : 1;
	}

};

#endif
