/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
map.cpp

  Map objects.
*/

#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#pragma hdrstop
#define _MAP_CPP_
#include "map.h"
#include "uavgs.h"
#include "sepstr.h"
#include "file.h"

Point2I pntMouseDn, pntOrig;
int mouseDrag = 0;

HMENU hmMapContext;

HFONT hfMapMarker;

HBRUSH hbrMapError, hbrMapHome;
HPEN hPenMapHome;

// degrees => map coords
Point2I transform(mapTransform& tr, Point2 p)
{
	Point2 pt;
	
	pt = (tr.scale * (p - tr.origin));
	return Point2I(int(pt.x), int(pt.y));
}
// degrees <= map coords
Point2 unTransform(mapTransform& tr, Point2I p)
{
	return ((Point2(p.x, p.y) / tr.scale) + tr.origin);
}

// map coords => view coords
Point2I transform(mapView& tr, Point2I p)
{
	Point2I pt;
	
	pt = p - tr.origin;
	return Point2I(pt.x / tr.zoom, pt.y / tr.zoom);
}
// map coords <= view coords
Point2I unTransform(mapView& tr, Point2I p)
{
	Point2I pt;

	pt = Point2I(p.x * tr.zoom, p.y * tr.zoom);
	return pt + tr.origin;
}

double degFromDMS(int d, int m, int s)
{
	if (d >= 0)
		return double(d) + (double(m) / 60.0) + (double(s) / 3600.0);
	else
		return - (double(-d) + (double(m) / 60.0) + (double(s) / 3600.0));
}
double degFromDMS(char *str)
{
	char d[64], m[64], s[64];

	strcpy(d, str);
	SepStr(d, ' ', d, m);
	SepStr(m, ' ', m, s);

	return degFromDMS(atoi(d), atoi(m), atoi(s));
}

void degMakeDMS(double deg, char *str)
{
	int d, m, s;
	double ms;

	deg = fabs(deg);

	d = (int) floor(deg);
	ms = 60.0 * (deg - double(d));

	m = (int) floor(ms);
	s = (int) fmod(60.0 * (ms - double(m)), 60.0);
	
	sprintf(str, "%03i %02i\' %02i\"", d, m, s);
}

COLORREF getColour(char *clr)
{
	char str[256];

	if (!stricmp(clr, "black"))
		return RGB(  0,   0,   0);
	if (!stricmp(clr, "darkgrey") || !stricmp(clr, "darkgray"))
		return RGB(128, 128, 128);
	if (!stricmp(clr, "lightgrey") || !stricmp(clr, "lightgray"))
		return RGB(192, 192, 192);
	if (!stricmp(clr, "white"))
		return RGB(255, 255, 255);

	if (!stricmp(clr, "red"))
		return RGB(192,   0,   0);
	if (!stricmp(clr, "green"))
		return RGB(  0, 128,   0);
	if (!stricmp(clr, "blue"))
		return RGB(  0,   0, 255);

	sprintf(str, "!Warning: Unknown colour %s", clr);
	message(str);
	return RGB(0, 0, 0);
}

// Returns the coordinates (in degrees) of the specified object
Point2 getCoords(char *name)
{
	char str[256];
	Point2 p;

	sprintf(str, "%s.latitude", name);
	p.y = getVar(str)->getNumber();
	sprintf(str, "%s.longitude", name);
	p.x = getVar(str)->getNumber();

	return p;
}

// Sets the coordinates (in degrees) of the specified object
void setCoords(char *name, Point2 p)
{
	char str[256];

	sprintf(str, "%s.latitude", name);
	getVar(str)->set(p.y);
	sprintf(str, "%s.longitude", name);
	getVar(str)->set(p.x);
}

void drawMapMarker(HDC hDC, Point2I p, char *name)
{
	POINT pts[3];
	RECT r, r2;

	// get text size
	memset(&r, 0, sizeof(r));
	DrawText(hDC, name, strlen(name), &r, DT_CALCRECT | DT_NOPREFIX | DT_SINGLELINE);
	r2.left   = p.x + 16;
	r2.top    = -8 + p.y - 24 - r.bottom;
	r2.right  =  8 + p.x + 16 + r.right;
	r2.bottom = p.y - 24;

	// draw marker
	pts[0].x = p.x;
	pts[0].y = p.y;

	pts[1].x = p.x + 16;
	pts[1].y = p.y - 24;
	
	pts[2].x = p.x + 16 + r.right;
	pts[2].y = p.y - 24;

	Polygon(hDC, pts, sizeof(pts) / sizeof(pts[0]));

	// draw text box
	SelectObject(hDC, (HBRUSH) GetStockObject(WHITE_BRUSH));
	Rectangle(hDC, r2.left, r2.top, r2.right, r2.bottom);

	// draw text
	DrawText(hDC, name, strlen(name), &r2, 
		DT_CENTER | DT_VCENTER | DT_NOPREFIX | DT_SINGLELINE | DT_NOCLIP);
}
void drawMap(HWND handle, HDC hDC)
{
	CWndFrameObj *wfo;
	CMap *map;
	CMapTile *tile;
	RECT rc;
	HDC hMem;
	Point2I p, center;
	int tx,ty;
	Point2I tMin, tMax;
	Point2I zoomSz;

	wfo = (CWndFrameObj*) GetProp(handle, "WFO");
	if (!wfo)
		return;
	map = wfo->map.map;
	if (!map)
		return;

	GetClientRect(handle, &rc);
	
	// erase background
	// (missing tiles show through)
	SetBkMode(hDC, OPAQUE);
	SetBkColor(hDC, RGB(255,255,255));
	FillRect(hDC, &rc, hbrMapError);

	// draw map tiles
	center = Point2I(rc.right / 2, rc.bottom / 2);
	tMin = unTransform(map->view, Point2I(0, 0) - center) / map->tileSz;
	tMax = unTransform(map->view, Point2I(rc.right, rc.bottom) - center) / map->tileSz;

	zoomSz = map->tileSz / map->view.zoom;

	for (tx = tMin.x; tx <= tMax.x; tx++)
	{
		for (ty = tMin.y; ty <= tMax.y; ty++)
		{
			tile = map->getTile(tx, ty);
			if (tile)
			{
				p = transform(map->view, Point2I(tx, ty) * map->tileSz) + center;
				hMem = CreateCompatibleDC(hDC);
				SelectObject(hMem, tile->hBmp);
				
				if (map->view.zoom == 1)
				{
					BitBlt(hDC, p.x, p.y, map->tileSz.x, map->tileSz.y, hMem, 0,0, SRCCOPY);
				} else {
					StretchBlt(hDC, p.x, p.y, zoomSz.x, zoomSz.y,
						hMem, 0,0, map->tileSz.x, map->tileSz.y, SRCCOPY);
				}
				/*
					Deleting the DC before the tile may possibly be deleted helps reduce
					memory leaks - even if the tile isn't selected at the time. Win95 sux.
				*/
				DeleteDC(hMem);
			}
		}
	}

	// draw lines
	map->drawLines(hDC, rc.right / 2, rc.bottom / 2);

	// get ready to draw markers
	SetBkMode(hDC, TRANSPARENT);
	SetTextColor(hDC, RGB(0,0,0));
	SelectObject(hDC, hfMapMarker);

	// draw "home" marker
	SelectObject(hDC, hbrMapHome);
	SelectObject(hDC, hPenMapHome);

	p = transform(map->view, transform(map->tr, getCoords("home"))) + center;
	drawMapMarker(hDC, p, "Home");

	// draw UAV marker
}
void mapMouseLDown(HWND handle, Point2I pos)
{
	CWndFrameObj *wfo;
	CMap *map;

	wfo = (CWndFrameObj*) GetProp(handle, "WFO");
	if (!wfo)
		return;
	map = wfo->map.map;
	if (!map)
		return;

	pntOrig = map->view.origin;
	pntMouseDn = pos;
	mouseDrag = 1;
	SetCapture(handle);
}
void mapMouseLUp(HWND handle, Point2I pos)
{
	CWndFrameObj *wfo;
	CMap *map;

	wfo = (CWndFrameObj*) GetProp(handle, "WFO");
	if (!wfo)
		return;
	map = wfo->map.map;
	if (!map)
		return;

	if (!mouseDrag)
		return;

	map->view.origin = pntOrig + ((pntMouseDn - pos) * map->view.zoom);
	InvalidateRect(handle, 0, 0);
	UpdateWindow(handle);

	mouseDrag = 0;
	ReleaseCapture();
}
void mapMouseMove(HWND handle, Point2I pos)
{
	CWndFrameObj *wfo;
	CMap *map;
	Point2 pt;
	RECT rc;
	char str[256];

	wfo = (CWndFrameObj*) GetProp(handle, "WFO");
	if (!wfo)
		return;
	map = wfo->map.map;
	if (!map)
		return;

	if (!mouseDrag)
	{
		// Show coordinates in status bar
		GetClientRect(handle, &rc);
		pos = pos - Point2I(rc.right / 2, rc.bottom / 2);
		pt = unTransform(map->tr, unTransform(map->view, pos));
		
		strcpy(str, (pt.y > 0) ? "N " : "S " );
		degMakeDMS(pt.y, str + 2);
		SendMessage(hWndStatus, SB_SETTEXT, 1, (LPARAM) str);

		strcpy(str, (pt.x > 0) ? "E " : "W " );
		degMakeDMS(pt.x, str + 2);
		SendMessage(hWndStatus, SB_SETTEXT, 2, (LPARAM) str);
		return;
	}

	map->view.origin = pntOrig + ((pntMouseDn - pos) * map->view.zoom);
	InvalidateRect(handle, 0, 0);
	UpdateWindow(handle);
}

LRESULT CALLBACK wpMap(HWND handle, UINT msg,
					   WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hDC, hMem;
	HBITMAP hBmp;
	RECT rc;
	CWndFrameObj *wfo;
	CMap *map;
	Point2 pt;
	POINT pnt;

	switch (msg)
	{
	case WM_CLOSE:
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);
		break;
		
	case WM_SIZE:
		// need a new backbuffer
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);

		// redraw
		InvalidateRect(handle, 0, 1);
		UpdateWindow(handle);
		break;
	
	case WM_CREATE:
		break;

	case WM_PAINT:
		GetClientRect(handle, &rc);

		hDC = BeginPaint(handle, &ps);
		hMem = CreateCompatibleDC(hDC);

		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (!hBmp)
		{			
			hBmp = CreateCompatibleBitmap(hDC, rc.right, rc.bottom);
			SetProp(handle, "BACKBUF", (HANDLE) hBmp);
		}
		SelectObject(hMem, hBmp);

		drawMap(handle, hMem);
		BitBlt(hDC, 0,0, rc.right, rc.bottom, hMem, 0,0, SRCCOPY);

		DeleteDC(hMem);
		EndPaint(handle, &ps);
		break;

	case WM_LBUTTONDOWN:
		/* note:
		 if we have capture the coordinates may go negative.
		 LOWORD is unsigned and 16 bits and int is signed and 32 bits,
		 so we have to cast to signed and 16 bits before extending to int */
		mapMouseLDown(handle, Point2I((short)LOWORD(lParam), (short)HIWORD(lParam)));
		break;

	case WM_MOUSEMOVE:
		mapMouseMove(handle, Point2I((short)LOWORD(lParam), (short)HIWORD(lParam)));
		break;

	case WM_LBUTTONUP:
		mapMouseLUp(handle, Point2I((short)LOWORD(lParam), (short)HIWORD(lParam)));
		break;

	case WM_RBUTTONDOWN:
		if (mouseDrag) 
			break;

		// Context menu
		pnt.x = LOWORD(lParam);
		pnt.y = HIWORD(lParam);
		ClientToScreen(handle, &pnt);
		TrackPopupMenuEx(hmMapContext, 
			TPM_LEFTALIGN | TPM_TOPALIGN | TPM_RIGHTBUTTON,
			pnt.x, pnt.y, handle, 0);
		break;

	case WM_COMMAND:
		wfo = (CWndFrameObj*) GetProp(handle, "WFO");
		if (!wfo)
			break;
		map = wfo->map.map;
		if (!map)
			break;
		
		switch (LOWORD(wParam))
		{
		// Context menu
		case 1001:	// Zoom In
			if (map->view.zoom > 1)
				map->view.zoom /= 2;
			InvalidateRect(handle, 0, 0);
			UpdateWindow(handle);
			break;
		case 1002:	// Zoom Out
			map->view.zoom *= 2;
			InvalidateRect(handle, 0, 0);
			UpdateWindow(handle);
			break;
		case 1011:	// Find Home
			map->viewCenter(pt = getCoords("home"));
			if (pt.LengthSquared() < 1 /* highly unlikely */)
				message("!Warning: Home position unknown");
			InvalidateRect(handle, 0, 0);
			UpdateWindow(handle);
			break;
		case 1012:	// Find UAV
			pt = getCoords("uav");
			if (pt.LengthSquared() < 1 /* highly unlikely */)
				message("!Warning: UAV position unknown");
			else {
				map->viewCenter(pt);
				InvalidateRect(handle, 0, 0);
				UpdateWindow(handle);
			}
			break;
		}
		break;
		
	default:
			return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}

LRESULT CALLBACK wpMiniMap(HWND handle, UINT msg,
						   WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;

	switch (msg)
	{
	case WM_CLOSE:
		break;
		
	case WM_SIZE:		
		break;
	
	case WM_CREATE:
		break;

	case WM_PAINT:
		BeginPaint(handle, &ps);
		EndPaint(handle, &ps);
		break;

	default:
		return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}


int regClassesMap()
{
	WNDCLASS wc;
	LOGFONT lf;
	HDC hDC;

	// Load/create misc junk
	hmMapContext = GetSubMenu(LoadMenu(hInstance, "MAP"), 0);

	memset(&lf, 0, sizeof(lf));
	lf.lfCharSet		= ANSI_CHARSET;
	lf.lfOutPrecision	= OUT_TT_PRECIS;
	lf.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
	lf.lfQuality		= DEFAULT_QUALITY;
	lf.lfPitchAndFamily	= DEFAULT_PITCH | FF_DONTCARE;
	lf.lfWeight			= FW_BOLD;
	strcpy(lf.lfFaceName, "Arial");

	hDC = GetDC(GetDesktopWindow());
	lf.lfHeight			= -MulDiv(10 /* points */, GetDeviceCaps(hDC, LOGPIXELSY), 72);
	ReleaseDC(GetDesktopWindow(), hDC);

	hfMapMarker = CreateFontIndirect(&lf);

	// Create pens / brushes
	hbrMapError = CreateHatchBrush(HS_CROSS, RGB(128, 0, 0));

	hPenMapHome = CreatePen(PS_SOLID, 0, RGB(0, 0, 128));
	hbrMapHome = CreateSolidBrush(RGB(0, 0, 128));

	// Map
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpMap;
	wc.hInstance	 = hInstance;
	wc.hCursor		 = LoadCursor(NULL, IDC_CROSS);
	wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
	wc.lpszClassName = "UAVGSmap";
	if (!RegisterClass(&wc))
		return 0;

	// Mini map
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpMiniMap;
	wc.hInstance	 = hInstance;
	wc.hCursor		 = LoadCursor(NULL, IDC_CROSS);
	wc.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);
	wc.lpszClassName = "UAVGSmmap";
	if (!RegisterClass(&wc))
		return 0;

	return 1;
}

/*
 
   CMap

  */
CMap::CMap()
{
	int i;

	view.zoom = 1;

	tileSz = Point2I(1, 1);

	nTiles = 16;
	for (i = 0; i < nTiles; i++)
		tile[i] = 0;

	lines.init();

	load(mapFilename);
	viewCenter(getCoords("home"));
}

CMap::~CMap()
{
	int i;

	for (i = 0; i < nTiles; i++)
		if (tile[i])
			delete tile[i];

	lines.destroy();
}

int CMap::load(char *fn)
{
	CFileR f;
	char str[256], a[256], b[256];
	CMapLine *line;
	Point2 pa,pb;

	if (!f.open(fn))
	{
		sprintf(str, "Could not load map %s", fn);
		error(str);

		tileSz = Point2I(1000, 1000);
		tr.origin = mapTL = mapBR = Point2(0.0);
		tr.scale = 1000.0;
		mapX = mapY = 0;
		tileFn[0] = 0;

		return 0;
	}

	sprintf(str, "Map %s contains invalid data", fn);

	// read tile filename string
	if (!ReadLine(&f, tileFn))
	{
		error(str);
		return 0;
	}
	
	// read tile size
	if (!ReadLine(&f, a))
	{
		error(str);
		return 0;
	}
	SepStr(a, 'x', a, b);
	tileSz = Point2I(atoi(a), atoi(b));
	if (!tileSz.x || !tileSz.y)
	{
		error(str);
		return 0;
	}

	// read top left coords (N/S then E/W)
	if (!ReadLine(&f, a))
	{
		error(str);
		return 0;
	}
	if (!ReadLine(&f, b))
	{
		error(str);
		return 0;
	}
	mapTL = Point2(atof(b), atof(a));

	// read map size
	if (!ReadLine(&f, a))
	{
		error(str);
		return 0;
	}
	SepStr(a, 'x', a, b);
	mapX = atoi(a);
	mapY = atoi(b);
	if (!mapX || !mapY)
	{
		error(str);
		return 0;
	}

	// read bottom right coords (N/S then E/W)
	if (!ReadLine(&f, a))
	{
		error(str);
		return 0;
	}
	if (!ReadLine(&f, b))
	{
		error(str);
		return 0;
	}
	mapBR = Point2(atof(b), atof(a));

	tr.origin = mapTL;
	tr.scale = Point2(mapX, mapY) / (mapBR - mapTL);

	// read default home coords (N/S then E/W)
	if (!ReadLine(&f, a))
	{
		error(str);
		return 0;
	}
	if (!ReadLine(&f, b))
	{
		error(str);
		return 0;
	}
	setCoords("home", Point2(atof(b), atof(a)));

	// read objects (if any)
	while (ReadLine(&f, a))
	{
		SepStr(a, ':', a, b);
		if (!stricmp(a, "line"))
		{
			line = new CMapLine;

			// "line" ":" colour " " startlat "," startlon ">" endlat "," endlon
			SepStr(b, ' ', a, b);
			line->clr = getColour(a);

			SepStr(b, ',', a, b);
			pa.y = degFromDMS(a);
			SepStr(b, '>', a, b);
			pa.x = degFromDMS(a);			

			SepStr(b, ',', a, b);
			pb.y = degFromDMS(a);
			pb.x = degFromDMS(b);

			// convert points to map space
			line->a = transform(tr, pa);
			line->b = transform(tr, pb);

			lines.append(line);
		}
	}

	f.close();
	return 1;
}

void CMap::viewCenter(Point2 pt)
{
	view.origin = transform(tr, pt);
}

CMapTile* CMap::getTile(int tx, int ty)
{
	int i;
	int tileLRU;
	DWORD tileLRUtime;
	char str[256];

	tileLRU = 0;
	tileLRUtime = 0;
	for (i = 0; i < nTiles; i++)
	{
		if (tile[i])
		{
			if (tile[i]->match(tx, ty))
			{
				tile[i]->touch();
				return tile[i];
			}

			if (tile[i]->tcMRU < tileLRUtime)
			{
				tileLRU = i;
				tileLRUtime = tile[i]->tcMRU;
			}
		} else {
			// an empty space is even better than a tile 
			// that hasn't been used in a while
			tileLRU = i;
			tileLRUtime = 0;
		}
	}

	// replace least recently used tile
	if (tile[tileLRU])
		delete tile[tileLRU];

	sprintf(str, tileFn, tx, ty);
	if ((tx * tileSz.x < mapX) && (ty * tileSz.y < mapY))
		tile[tileLRU] = new CMapTile(tx, ty, str);
	else
		tile[tileLRU] = new CMapTile(tx, ty, 0);	// out of range

	if (tile[tileLRU]->hBmp)
		return tile[tileLRU];

	// tile is missing
	delete tile[tileLRU];
	tile[tileLRU] = 0;

	return 0;
}

void CMap::drawLines(HDC hDC, int xoff, int yoff)
{
	lines.home();
	while (!lines.pastend())
	{
		lines.data()->draw(hDC, view, xoff, yoff);
		
		++lines;
	}
}

/*
 
   CMapTile

  */
CMapTile::CMapTile()
{
	x = y = 0;
	hBmp = 0;
	tcMRU = 0;
}
CMapTile::CMapTile(int _x, int _y, char *fn)
{
	char str[256];

	x = _x; 
	y = _y;

	hBmp = 0;	

	if ((x < 0) || (y < 0) || !fn)
	{
		// invalid
		touch();
		return;
	}

	SetLastError(0);	// sometimes LoadImage is naughty and doesn't set the error code!
	hBmp = (HBITMAP) LoadImage(hInstance, fn, IMAGE_BITMAP, 0,0, 
		LR_LOADFROMFILE// | LR_CREATEDIBSECTION
		/* LR_CREATEDIBSECTION saves memory if the map has a low number of bits per pixel */);

	if (!hBmp)
	{
		sprintf(str, "!Warning: Error loading map tile (%i, %i) - %s (GetLastError() = %i)",
			_x, _y, fn, GetLastError());
		message(str);
	}

	touch();
}
CMapTile::~CMapTile()
{
	if (hBmp)
	{
		if (!DeleteObject(hBmp))
		{
			message("!Warning: Map tile not deleted");
		}
	}
}

void CMapTile::touch()
{
	tcMRU = GetTickCount();
}
bool CMapTile::match(int _x, int _y)
{
	return (x == _x) && (y == _y);
}

/*

  CMapLine

 */
CMapLine::CMapLine()
{
	a = b = Point2I(0, 0);
	clr = RGB(0, 0, 0);
}
CMapLine::CMapLine(Point2I _a, Point2I _b, COLORREF _clr)
{
	a = _a;
	b = _b;
	clr = _clr;
}
CMapLine::~CMapLine()
{
}
void CMapLine::draw(HDC hDC, mapView& view, int xoff, int yoff)
{
	Point2I at, bt;
	HPEN hP;

	hP = CreatePen(PS_SOLID, 2, clr);
	SelectObject(hDC, hP);

	at = transform(view, a);
	bt = transform(view, b);

	MoveToEx(hDC, at.x + xoff, at.y + yoff, 0);
	LineTo(hDC, bt.x + xoff, bt.y + yoff);

	SelectObject(hDC, GetStockObject(BLACK_PEN));
	DeleteObject(hP);
}
