/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
map.h

  Map objects.
*/

#ifndef _MAP_H_
#define _MAP_H_

#ifndef _MAP_CPP_
#define _MAP_CPP_ extern
#endif

#include <windows.h>

class CMapLine;
class CMapTile;
class CMap;
struct mapTransform;

#include "point.h"
#include "linkedlist.h"

struct mapTransform {
	Point2 origin;
	Point2 scale;
};
struct mapView {
	Point2I origin;
	int zoom;
};

class CMapLine
{
public:
	COLORREF clr;
	Point2I a,b;		// endpoints in map space

	CMapLine();
	CMapLine(Point2I _a, Point2I _b, COLORREF _clr);
	~CMapLine();

	void draw(HDC hDC, mapView& view, int xoff = 0, int yoff = 0);
};
class CMapTile
{
public:
	HBITMAP hBmp;
	int x, y;
	DWORD tcMRU;	// time last accessed

	CMapTile();
	CMapTile(int _x, int _y, char *fn);
	~CMapTile();

	void touch();	// update access time
	bool match(int _x, int _y);
};
class CMap
{
public:
	char tileFn[256];
	Point2 mapTL, mapBR;
	int mapX, mapY;

	CMapTile *tile[256];
	int nTiles;
	Point2I tileSz;

	CLList<CMapLine*> lines;

	mapTransform tr;
	mapView view;
	
	CMap();
	~CMap();

	int load(char *fn);
	void viewCenter(Point2 pt);
	CMapTile * getTile(int tx, int ty);
	void drawLines(HDC hDC, int xoff = 0, int yoff = 0);
};

int regClassesMap();

double degFromDMS(int d, int m, int s);
void degMakeDMS(double deg, char *str);

Point2 getCoords(char *name);
void setCoords(char *name, Point2 p);

#endif
