/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
miscctl.cpp

  Controls that didn't fit anywhere else.
*/

#include <stdio.h>
#include <windows.h>
#pragma hdrstop
#define _MISCCTL_CPP_
#include "miscctl.h"
#include "gauge.h"
#include "uavgs.h"
#include "packet.h"
#include "point.h"
#include "draw.h"

void drawLabel(HWND handle, HDC hDC)
{
	CWndFrameObj *wfo;
	ptTransform2 tr;
	RECT rc;
	Point2 pt;
	HFONT hf;

	wfo = (CWndFrameObj*) GetProp(handle, "WFO");
	if (!wfo)
		return;

	// Text size is relative to main window so all
	// the labels look the same
	GetClientRect(hWnd1, &rc);
	transformSquarify(tr, (double) rc.right, (double) rc.bottom);
	hf = drawMakeFont(hDC, tr, wfo->label.size);

	GetClientRect(handle, &rc);
	tr.origin = Point2(-rc.right / 2, -rc.bottom / 2) / tr.scale;

	// draw text
	SetTextColor(hDC, RGB(255,255,255));
	SetBkMode(hDC, TRANSPARENT);

	drawText(hDC, tr, unPolar(Point2(0, 0)), wfo->label.text);

	SelectObject(hDC, hfDefault);
	DeleteObject(hf);

}
LRESULT CALLBACK wpLabel(HWND handle, UINT msg,
						 WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hDC, hMem;
	HBITMAP hBmp;
	RECT rc;

	switch (msg)
	{
	case WM_CLOSE:
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);
		break;
		
	case WM_SIZE:
		// need a new backbuffer
		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (hBmp)
			DeleteObject(hBmp);
		SetProp(handle, "BACKBUF", 0);

		// redraw
		InvalidateRect(handle, 0, 1);
		UpdateWindow(handle);
		break;
	
	case WM_CREATE:
		break;

	case WM_PAINT:
		GetClientRect(handle, &rc);

		hDC = BeginPaint(handle, &ps);
		hMem = CreateCompatibleDC(hDC);

		hBmp = (HBITMAP) GetProp(handle, "BACKBUF");
		if (!hBmp)
		{			
			hBmp = CreateCompatibleBitmap(hDC, rc.right, rc.bottom);
			SetProp(handle, "BACKBUF", (HANDLE) hBmp);
		}
		SelectObject(hMem, hBmp);

		eraseGaugeBkg(handle, hMem, &rc);
		drawLabel(handle, hMem);

		BitBlt(hDC, 0,0, rc.right, rc.bottom, hMem, 0,0, SRCCOPY);

		DeleteDC(hMem);
		EndPaint(handle, &ps);
		break;

	default:
		return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}

int regClassesMisc()
{
	WNDCLASS wc;

	// Label
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpLabel;
	wc.hInstance	 = hInstance;
	wc.hCursor		 = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
	wc.lpszClassName = "UAVGSlabel";
	if (!RegisterClass(&wc))
		return 0;

	return 1;
}

