/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*

  MkMap
  Converts a bitmap into small tiles

*/

#include <windows.h>
#include <stdio.h>
#pragma hdrstop
#include "file.h"
#include "sepstr.h"

char fnInput[256], fnSource[256];

int x1, x2, y1, y2;

CFileR bmp;
CFileW out;

struct BitmapJunk {
	BITMAPFILEHEADER bfh;
	BITMAPINFOHEADER bih;
	RGBQUAD pal[16];
} bmphdr;

void bltToFile(CFileR &in, CFileW &out, int x, int y, int sx, int sy)
{
	BitmapJunk bmphdrOut;
	char buffer[65536]; // should be plenty ;)
	int scan;
	unsigned long newpos;
	long w;

	while (sx % 8) sx++; // dword padded apparently

	x -= x % 2; sx -= sx % 2;	// 4 bpp - and i'm NOT copying extra nibbles!

	memcpy(&bmphdrOut, &bmphdr, sizeof(bmphdr));
	bmphdrOut.bfh.bfSize = 118 + sx*sy / 2;
	bmphdrOut.bih.biWidth = sx;
	bmphdrOut.bih.biHeight = sy;
	bmphdrOut.bih.biSizeImage = 0;

	out.write(&bmphdrOut.bfh, sizeof(bmphdrOut.bfh));
	out.write(&bmphdrOut.bih, sizeof(bmphdrOut.bih));
	out.write(&bmphdrOut.pal, sizeof(bmphdrOut.pal));

	w = bmphdr.bih.biWidth;
	while (w % 8) w++; // dword padded apparently

	for (scan = 0; scan < sy; scan++)
	{
		newpos = SetFilePointer(in.hFile, 
			bmphdr.bfh.bfOffBits + ((bmphdr.bih.biHeight - (y + scan) - 1) * w + x) / 2,
			0, FILE_BEGIN);
		in.read(buffer, sx / 2);

		newpos = SetFilePointer(out.hFile, 
			bmphdrOut.bfh.bfOffBits + ((bmphdrOut.bih.biHeight - scan - 1) * sx / 2),
			0, FILE_BEGIN);
		out.write(buffer, sx / 2);
	}
}

int PASCAL WinMain(HINSTANCE hCur, HINSTANCE hPrev, LPSTR lpCmdLine, int nCmdShow)
{
	CFileR in;
	char str[256], a[256], b[256];
	int x, y, tileX, tileY;

	strcpy(fnInput, lpCmdLine);
	if (!in.open(fnInput))
	{
		x = GetLastError(); // for debugging
		MessageBox(0, "Error: cannot open .map file!", "MkMap", MB_ICONSTOP);
		return 0;
	}

	ReadLine(&in, fnSource);

	// Load bitmap
	if (!bmp.open(fnSource))
	{
		x = GetLastError(); // for debugging
		MessageBox(0, "Error: cannot open source bitmap!", "MkMap", MB_ICONSTOP);
		return 0;
	}

	// WOW this is unportable!
	// BITMAPINFOHEADER is obsolete but hey Paint seems to have used it
	// Also the palette - well what if it's not 4 bpp?
	bmp.read(&bmphdr.bfh, sizeof(bmphdr.bfh));
	bmp.read(&bmphdr.bih, sizeof(bmphdr.bih));
	bmp.read(&bmphdr.pal, sizeof(bmphdr.pal));

	// Load info
	ReadLine(&in, str);	// top left
	SepStr(str, ',', a, b);
	x1 = atoi(a);
	y1 = atoi(b);
	ReadLine(&in, str);	// coords
	ReadLine(&in, str);

	ReadLine(&in, str);	// bottom right
	SepStr(str, ',', a, b);
	x2 = atoi(a);
	y2 = atoi(b);
	ReadLine(&in, str);	// coords
	ReadLine(&in, str);

	RSepStr(fnSource, '.', fnSource, 0); // remove .bmp

	// Convert to tiles
	tileY = 0;
	for (y = y1; y < y2; y += 300)	
	{
		tileX = 0;
		for (x = x1; x < x2; x += 300)
		{
			sprintf(str, "%s tile %i,%i.bmp", fnSource, tileX, tileY);
			if (!out.open(str))
			{
				x = GetLastError(); // for debugging
				MessageBox(0, "Error: cannot open output bitmap!", "MkMap", MB_ICONSTOP);
				return 0;
			}

			bltToFile(bmp, out, x, y, 300, 300);

			out.close();

			tileX++;
		}
		tileY++;
	}

	bmp.close();
	in.close();

	MessageBox(0, "Map has been converted to tiles.", "MkMap", MB_ICONINFORMATION);

	return 123; // cuz i forget what it's supposed to return and i'm not looking it up
}
