/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
packet.h

  Telemetry packet processing.
*/

#ifndef _PACKET_H_
#define _PACKET_H_

#ifndef _PACKET_CPP_
#define _PACKET_CPP_ extern
#endif

class CVariable;
class CPacketItem;
class CPacketType;
class CTelemetry;

#include <windows.h>
#include "linkedlist.h"
#include "wndlayout.h"

int initTelemetry();
void stopTelemetry();

CVariable *getVar(char *name);
void deleteVars();

#define VART_NUMBER		1
#define VART_STRING		2
#define VART_GPSLAT		3
#define VART_GPSLON		4

class CVariable {
public:
	char name[64];
	int type;		// VART_*
	double valueNum;
	char valueStr[256];

	int nWindow;
	CWndFrameObj *wnd[256];

	CVariable();
	~CVariable();

	void set(double value);
	void set(char *value);
	double getNumber();
	char * getString();
	void addWindow(CWndFrameObj *w);
};

#define PKTI_NULL		0
#define	PKTI_MESSAGE	1
#define PKTI_BINARY		2
#define PKTI_STRING		3
#define PKTI_NUMBER		4
#define PKTI_GPSLAT		5
#define PKTI_GPSLON		6

#define PKTI_BINARY_MSB1	1	// else LSB first
#define PKTI_BINARY_SIGN	2	// else unsigned
#define PKTI_BINARY_16		4	// if no size specified then 8 bits
#define PKTI_BINARY_32		8

class CPacketItem {
public:
	int type;		// PKTI_*
	int details;	// usually PKTI_*_* flags
	CVariable *var;

	double scale;

	CPacketItem();
	CPacketItem(int _type, int _details, char *varName = 0);
	~CPacketItem();
};

class CPacketType {
public:
	char start[64];				// Beginning of packet
	int nItem;					// Number of items in packet
	CPacketItem *item[256];		// List of items in packet

	CPacketType();
	~CPacketType();

	int isMatch(char *packet);
};

class CTelemetry {
public:
	HANDLE hCom;				// COM port handle
	unsigned long comBaud;		// COM baud rate
	char comPort[256];			// COM1, COM2 etc
	char chrStart, chrEnd;		// Packet start/end characters

	int nPType;					// Number of packet types
	CPacketType *ptype[256];	// List of packet types

	void processPacket(char *packet);

	int initComPort(char *comPort);
	void stopComPort();
	int initTelemetry();
	void stopTelemetry();

	CTelemetry();
	~CTelemetry();
};

_PACKET_CPP_ CTelemetry *telemetry;

#endif
