/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
 Point.h

  Classes for working with points and vectors. Supports
  addition, subtraction, multiplication, and division.
 */
#ifndef _Point_H
#define _Point_H

#include <math.h>	// Square roots, etc.

class Point2 
{
public:
	union {
		struct
		{
			double x, y;
		};
		double values[2];
	};

	// Constructors and destructors
    Point2();
    Point2( const double _x, const double _y );
    Point2( const double values[2] );
    Point2( const double value );
    ~Point2();

	// Operators
    Point2 operator + ( const Point2 P ) const;
    Point2 operator - ( const Point2 P ) const;
    Point2 operator * ( const Point2 P ) const;
	Point2 operator / ( const Point2 P ) const;

	// Functions
	double Length() const;
	double LengthSquared() const;
};
typedef Point2 Vector2;

class Point3 
{
public:
	union {
		struct 
		{
			double x, y, z;
		};
		double values[3];
	};

	// Constructors and destructors
	Point3();
	Point3( const double _x, const double _y, const double _z );
	Point3( const double values[3] );
	Point3( const double value );
	Point3( Point2 P );
	~Point3();

	// Operators
	Point3 operator + ( const Point3 P ) const;
	Point3 operator - ( const Point3 P ) const;
	Point3 operator * ( const Point3 P ) const;
	Point3 operator / ( const Point3 P ) const;

	// Functions
	double Length() const;
	double LengthSquared() const;
};
typedef Point3 Vector3;

class Point2I
{
public:
	union {
		struct
		{
			int x, y;
		};
		int values[2];
	};

	// Constructors and destructors
    Point2I();
    Point2I( const int _x, const int _y );
    Point2I( const int values[2] );
    Point2I( const int value );
    ~Point2I();

	// Operators
    Point2I operator + ( const Point2I P ) const;
    Point2I operator - ( const Point2I P ) const;
    Point2I operator * ( const Point2I P ) const;
	Point2I operator / ( const Point2I P ) const;

	// Functions
	double Length() const;
	double LengthSquared() const;
};
typedef Point2I Vector2I;

double DotProduct( const Vector3 A, const Vector3 B );
Vector3 CrossProduct( const Vector3 A, const Vector3 B );

void Normalize( Vector3 *V);
Vector3 Normalize( Vector3 V );

struct ptTransform2 {
	Point2 scale, origin;
};
Point2 transform(ptTransform2& tr, Point2 p);
Point2 unTransform(ptTransform2& tr, Point2 p);

#endif

