/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
script.cpp

  Scripting stuff.
*/

#include <stdio.h>
#include <windows.h>
#pragma hdrstop
#define _SCRIPT_CPP_
#include "script.h"
#include "file.h"
#include "uavgs.h"
#include "packet.h"

#include "sepstr.h"

int getScriptLine(CFileR *scp, char *line)
{
	char curline[256], *lineptr;

	do {
		if (!ReadLine(scp, curline))
			return 0;
		
		// Remove any comments
		SepStr(curline, ';', curline, 0);

		// Skip past spaces and tabs
		lineptr = curline;
		while ((*lineptr == ' ') || (*lineptr == '\t')) 
			lineptr++;
	} while (!*lineptr);

	strcpy(line, lineptr);
	return 1;
}

/*
 
   windows{}

 */

int scriptLoadWndTextbox(CFileR *scp, char *myData, CWndFrame *parent)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CWndFrameObj *wnd;
	int multiLine;
	DWORD dw;

	wnd = new CWndFrameObj;
	parent->objects.append(wnd);

	multiLine = 0;

	// name ' ' x ',' y ' ' w 'x' h
	SepStr(myData, ' ', wnd->name, myData);
	SepStr(myData, ' ', a, myData);
	SepStr(a, ',', a, b);
	wnd->x = atof(a);
	wnd->y = atof(b);
	SepStr(myData, 'x', a, b);
	wnd->w = atof(a);
	wnd->h = atof(b);

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"textbox\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"textbox\"");
		return 0;
	}

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"textbox\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "padding"))
		{
			wnd->padding = atoi(data);
		} else if (!stricmp(cmd, "multiline")) {
			multiLine = 1;
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"textbox\"", cmd);
			error(str);
			return 0;
		}
	}

	wnd->createTextbox(multiLine);
	getVar(wnd->name)->addWindow(wnd);
	if (!stricmp(wnd->name, "Messages"))
	{
		hWndMsg = wnd->handle;

		dw = 48;
		SendMessage(hWndMsg, EM_SETTABSTOPS, 1, (LPARAM) &dw);
	}

	return 1;
}
int scriptLoadWndAltimeter(CFileR *scp, char *myData, CWndFrame *parent)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CWndFrameObj *wnd;

	wnd = new CWndFrameObj;
	parent->objects.append(wnd);

	// name ' ' x ',' y ' ' w 'x' h
	SepStr(myData, ' ', wnd->name, myData);
	SepStr(myData, ' ', a, myData);
	SepStr(a, ',', a, b);
	wnd->x = atof(a);
	wnd->y = atof(b);
	SepStr(myData, 'x', a, b);
	wnd->w = atof(a);
	wnd->h = atof(b);

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"altimeter\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"altimeter\"");
		return 0;
	}

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"altimeter\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "padding"))
		{
			wnd->padding = atoi(data);
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"altimeter\"", cmd);
			error(str);
			return 0;
		}
	}

	wnd->createAltimeter();
	getVar(wnd->name)->addWindow(wnd);
	if (!stricmp(wnd->name, "Messages"))
		hWndMsg = wnd->handle;

	return 1;
}
int scriptLoadWndCompass(CFileR *scp, char *myData, CWndFrame *parent)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CWndFrameObj *wnd;

	wnd = new CWndFrameObj;
	parent->objects.append(wnd);

	// name ' ' x ',' y ' ' w 'x' h
	SepStr(myData, ' ', wnd->name, myData);
	SepStr(myData, ' ', a, myData);
	SepStr(a, ',', a, b);
	wnd->x = atof(a);
	wnd->y = atof(b);
	SepStr(myData, 'x', a, b);
	wnd->w = atof(a);
	wnd->h = atof(b);

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"compass\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"compass\"");
		return 0;
	}

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"compass\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "padding"))
		{
			wnd->padding = atoi(data);
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"compass\"", cmd);
			error(str);
			return 0;
		}
	}

	wnd->createCompass();
	getVar(wnd->name)->addWindow(wnd);
	if (!stricmp(wnd->name, "Messages"))
		hWndMsg = wnd->handle;

	return 1;
}
int scriptLoadWndRoundGauge(CFileR *scp, char *myData, CWndFrame *parent)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CWndFrameObj *wnd;
	wfodRoundGauge wfod;

	wnd = new CWndFrameObj;
	parent->objects.append(wnd);
	memset(&wfod, 0, sizeof(wfod));
	wfod.minAngle = 0.0;
	wfod.minValue = 0.0;
	wfod.maxAngle = 360.0;
	wfod.maxValue = 1.0;

	// name ' ' x ',' y ' ' w 'x' h
	SepStr(myData, ' ', wnd->name, myData);
	SepStr(myData, ' ', a, myData);
	SepStr(a, ',', a, b);
	wnd->x = atof(a);
	wnd->y = atof(b);
	SepStr(myData, 'x', a, b);
	wnd->w = atof(a);
	wnd->h = atof(b);

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"roundgauge\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"roundgauge\"");
		return 0;
	}

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"roundgauge\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "padding"))
		{
			wnd->padding = atoi(data);
		} else if (!stricmp(cmd, "minvalue")) {
			wfod.minValue = atof(data);
		} else if (!stricmp(cmd, "maxvalue")) {
			wfod.maxValue = atof(data);
		} else if (!stricmp(cmd, "minangle")) {
			wfod.minAngle = atof(data);
		} else if (!stricmp(cmd, "maxangle")) {
			wfod.maxAngle = atof(data);
		} else if (!stricmp(cmd, "warninglow")) {
			wfod.warnLo = atof(data);
			wfod.doWarnLo = 1;
		} else if (!stricmp(cmd, "warninghigh")) {
			wfod.warnHi = atof(data);
			wfod.doWarnHi = 1;
		} else if (!stricmp(cmd, "errorlow")) {
			wfod.errorLo = atof(data);
			wfod.doErrorLo = 1;
		} else if (!stricmp(cmd, "errorhigh")) {
			wfod.errorHi = atof(data);
			wfod.doErrorHi = 1;
		} else if (!stricmp(cmd, "tickspacing")) {
			wfod.tickSpace = atof(data);
			wfod.doTicks = 1;
		} else if (!stricmp(cmd, "smalltickspacing")) {
			wfod.tickSpace2 = atof(data);
			wfod.doTicks2 = 1;
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"roundgauge\"", cmd);
			error(str);
			return 0;
		}
	}

	wnd->createRoundGauge();
	memcpy(&wnd->roundGauge, &wfod, sizeof(wfod));

	getVar(wnd->name)->addWindow(wnd);
	if (!stricmp(wnd->name, "Messages"))
		hWndMsg = wnd->handle;

	return 1;
}
int scriptLoadWndLCD(CFileR *scp, char *myData, CWndFrame *parent)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CWndFrameObj *wnd;
	wfodLCD wfod;

	wnd = new CWndFrameObj;
	parent->objects.append(wnd);
	memset(&wfod, 0, sizeof(wfod));
	wfod.nDigits = 1;
	wfod.msDigit = 1.0;

	// name ' ' x ',' y ' ' w 'x' h
	SepStr(myData, ' ', wnd->name, myData);
	SepStr(myData, ' ', a, myData);
	SepStr(a, ',', a, b);
	wnd->x = atof(a);
	wnd->y = atof(b);
	SepStr(myData, 'x', a, b);
	wnd->w = atof(a);
	wnd->h = atof(b);

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"lcd\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"lcd\"");
		return 0;
	}

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"lcd\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "padding"))
		{
			wnd->padding = atoi(data);
		} else if (!stricmp(cmd, "digits")) {
			wfod.nDigits = atoi(data);
		} else if (!stricmp(cmd, "maxdigit")) {
			wfod.msDigit = atof(data);
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"lcd\"", cmd);
			error(str);
			return 0;
		}
	}

	wnd->createLCD();
	memcpy(&wnd->lcd, &wfod, sizeof(wfod));

	getVar(wnd->name)->addWindow(wnd);
	if (!stricmp(wnd->name, "Messages"))
		hWndMsg = wnd->handle;

	return 1;
}
int scriptLoadWndLCDText(CFileR *scp, char *myData, CWndFrame *parent)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CWndFrameObj *wnd;
	wfodLCDText wfod;

	wnd = new CWndFrameObj;
	parent->objects.append(wnd);
	memset(&wfod, 0, sizeof(wfod));
	wfod.size = 1;

	// name ' ' x ',' y ' ' w 'x' h
	SepStr(myData, ' ', wnd->name, myData);
	SepStr(myData, ' ', a, myData);
	SepStr(a, ',', a, b);
	wnd->x = atof(a);
	wnd->y = atof(b);
	SepStr(myData, 'x', a, b);
	wnd->w = atof(a);
	wnd->h = atof(b);

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"lcdtext\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"lcdtext\"");
		return 0;
	}

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"lcdtext\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "padding"))
		{
			wnd->padding = atoi(data);
		} else if (!stricmp(cmd, "size")) {
			wfod.size = atoi(data);
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"lcdtext\"", cmd);
			error(str);
			return 0;
		}
	}

	wnd->createLCDText();
	memcpy(&wnd->lcdtext, &wfod, sizeof(wfod));

	getVar(wnd->name)->addWindow(wnd);
	if (!stricmp(wnd->name, "Messages"))
		hWndMsg = wnd->handle;

	return 1;
}
int scriptLoadWndMap(CFileR *scp, char *myData, CWndFrame *parent)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CWndFrameObj *wnd;
	wfodMap wfod;

	wnd = new CWndFrameObj;
	parent->objects.append(wnd);
	memset(&wfod, 0, sizeof(wfod));
	wfod.map = new CMap;

	// name ' ' x ',' y ' ' w 'x' h
	SepStr(myData, ' ', wnd->name, myData);
	SepStr(myData, ' ', a, myData);
	SepStr(a, ',', a, b);
	wnd->x = atof(a);
	wnd->y = atof(b);
	SepStr(myData, 'x', a, b);
	wnd->w = atof(a);
	wnd->h = atof(b);

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"map\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"map\"");
		return 0;
	}

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"map\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "padding"))
		{
			wnd->padding = atoi(data);
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"map\"", cmd);
			error(str);
			return 0;
		}
	}

	wnd->createMap();
	memcpy(&wnd->map, &wfod, sizeof(wfod));

	getVar(wnd->name)->addWindow(wnd);
	if (!stricmp(wnd->name, "Messages"))
		hWndMsg = wnd->handle;

	return 1;
}
int scriptLoadWndLabel(CFileR *scp, char *myData, CWndFrame *parent)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CWndFrameObj *wnd;
	wfodLabel wfod;

	wnd = new CWndFrameObj;
	parent->objects.append(wnd);

	wfod.size = 0.1;
	strcpy(wfod.text, "(no text)");

	// name ' ' x ',' y ' ' w 'x' h
	SepStr(myData, ' ', wnd->name, myData);
	SepStr(myData, ' ', a, myData);
	SepStr(a, ',', a, b);
	wnd->x = atof(a);
	wnd->y = atof(b);
	SepStr(myData, 'x', a, b);
	wnd->w = atof(a);
	wnd->h = atof(b);

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"label\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"label\"");
		return 0;
	}

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"label\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "padding"))
		{
			wnd->padding = atoi(data);
		} else if (!stricmp(cmd, "text")) {
			strcpy(wfod.text, data);
		} else if (!stricmp(cmd, "size")) {
			wfod.size = atof(data);
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"label\"", cmd);
			error(str);
			return 0;
		}
	}

	wnd->createLabel();
	memcpy(&wnd->label, &wfod, sizeof(wfod));

	getVar(wnd->name)->addWindow(wnd);
	if (!stricmp(wnd->name, "Messages"))
		hWndMsg = wnd->handle;

	return 1;
}

int scriptLoadWndFrame(CFileR *scp, char *myData, CWndFrame *parent = 0)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CWndFrame *frame;

	frame = new CWndFrame;
	if (parent)
		parent->frames.append(frame);
	else
		wndLayout.frames.append(frame);

	// x ',' y ' ' w 'x' h
	SepStr(myData, ' ', a, myData);
	SepStr(a, ',', a, b);
	frame->x = atof(a);
	frame->y = atof(b);
	SepStr(myData, 'x', a, b);
	frame->w = atof(a);
	frame->h = atof(b);

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"frame\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"frame\"");
		return 0;
	}

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"frame\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "frame"))
		{
			// recursion rulez
			if (!scriptLoadWndFrame(scp, data, frame))
				return 0;
		} else if (!stricmp(cmd, "textbox")) {
			if (!scriptLoadWndTextbox(scp, data, frame))
				return 0;
		} else if (!stricmp(cmd, "altimeter")) {
			if (!scriptLoadWndAltimeter(scp, data, frame))
				return 0;
		} else if (!stricmp(cmd, "compass")) {
			if (!scriptLoadWndCompass(scp, data, frame))
				return 0;
		} else if (!stricmp(cmd, "roundgauge")) {
			if (!scriptLoadWndRoundGauge(scp, data, frame))
				return 0;
		} else if (!stricmp(cmd, "lcd")) {
			if (!scriptLoadWndLCD(scp, data, frame))
				return 0;
		} else if (!stricmp(cmd, "lcdtext")) {
			if (!scriptLoadWndLCDText(scp, data, frame))
				return 0;
		} else if (!stricmp(cmd, "map")) {
			if (!scriptLoadWndMap(scp, data, frame))
				return 0;
		} else if (!stricmp(cmd, "label")) {
			if (!scriptLoadWndLabel(scp, data, frame))
				return 0;
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"frame\"", cmd);
			error(str);
			return 0;
		}
	}

	return 1;
}

int scriptLoadWindows(CFileR *scp)
{
	char line[256], str[256];
	char cmd[256], data[256];

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"windows\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"windows\"");
		return 0;
	}
	
	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"windows\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "frame"))
		{
			if (!scriptLoadWndFrame(scp, data))
				return 0;
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"windows\"", cmd);
			error(str);
			return 0;
		}
	}

	return 1;
}

/*
 
   telemetry {}

*/
int scriptLoadTlmPacket(CFileR *scp, char *myData)
{
	char line[256], str[256];
	char cmd[256], data[256];
	char a[256], b[256];
	CPacketType *pkt;
	int i;

	if ((myData[0] != '\"') || !myData[1] || (myData[strlen(myData)-1] != '\"'))
	{
		sprintf(str,
			"Script Error: Bad \"packet\" data \"%s\" (expect packet \"string\")",
			myData);
		return 0;
	}

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"packet\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"packet\"");
		return 0;
	}

	// Set up CPacketType
	pkt = new CPacketType;
	telemetry->ptype[telemetry->nPType] = pkt;
	telemetry->nPType ++;

	myData[strlen(myData)-1] = 0;
	strcpy(pkt->start, myData + 1);

	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"packet\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "message"))
		{
			pkt->item[pkt->nItem] = new CPacketItem(PKTI_MESSAGE, 0);
			pkt->nItem ++;
		} else if (!stricmp(cmd, "binary")) {
			sprintf(str, "Script Error: Bad data \"%s\" in \"packet\" \"binary\"", data);

			SepStr(data, ' ', data, a);
			// data = name, a = options
			SepStr(a, ',', a, b);

			if (!stricmp(a, "unsigned_32"))
			{
				i = PKTI_BINARY_32;
				
				// msb or lsb first?
				if (!stricmp(b, "msbfirst"))
					i |= PKTI_BINARY_MSB1;
				else if (stricmp(b, "lsbfirst"))
				{
					strcat(str, " (must specify lsbfirst or msbfirst)");
					error(str);
					return 0;
				}
			} else if (!stricmp(a, "signed_32")) {
				i = PKTI_BINARY_32 | PKTI_BINARY_SIGN;

							// msb or lsb first?
				if (!stricmp(b, "msbfirst"))
					i |= PKTI_BINARY_MSB1;
				else if (stricmp(b, "lsbfirst"))
				{
					strcat(str, " (must specify lsbfirst or msbfirst)");
					error(str);
					return 0;
				}
			} else if (!stricmp(a, "unsigned_16")) {
				i = PKTI_BINARY_16;
				
				// msb or lsb first?
				if (!stricmp(b, "msbfirst"))
					i |= PKTI_BINARY_MSB1;
				else if (stricmp(b, "lsbfirst"))
				{
					strcat(str, " (must specify lsbfirst or msbfirst)");
					error(str);
					return 0;
				}
			} else if (!stricmp(a, "signed_16")) {
				i = PKTI_BINARY_16 | PKTI_BINARY_SIGN;

				// msb or lsb first?
				if (!stricmp(b, "msbfirst"))
					i |= PKTI_BINARY_MSB1;
				else if (stricmp(b, "lsbfirst"))
				{
					strcat(str, " (must specify lsbfirst or msbfirst)");
					error(str);
					return 0;
				}
			} else if (!stricmp(a, "unsigned_8")) {
				i = 0;
				if (strlen(b))
				{
					strcat(str, " (with 8 bits cannot specify lsbfirst or msbfirst)");
					error(str);
					return 0;
				}
			} else if (!stricmp(a, "signed_8")) {
				i = PKTI_BINARY_SIGN;
				if (strlen(b))
				{
					strcat(str, " (with 8 bits cannot specify lsbfirst or msbfirst)");
					error(str);
					return 0;
				}
			} else {
				error(str);
				return 0;
			}

			pkt->item[pkt->nItem] = new CPacketItem(PKTI_BINARY, i, data);
			pkt->nItem ++;
		} else if (!stricmp(cmd, "number")) {
			pkt->item[pkt->nItem] = new CPacketItem(PKTI_NUMBER, 0, data);
			pkt->nItem ++;
		} else if (!stricmp(cmd, "string")) {
			pkt->item[pkt->nItem] = new CPacketItem(PKTI_STRING, 0, data);
			pkt->nItem ++;
		} else if (!stricmp(cmd, "gpslat")) {
			pkt->item[pkt->nItem] = new CPacketItem(PKTI_GPSLAT, 0, data);
			pkt->nItem ++;
		} else if (!stricmp(cmd, "gpslon")) {
			pkt->item[pkt->nItem] = new CPacketItem(PKTI_GPSLON, 0, data);
			pkt->nItem ++;
		} else if (!stricmp(cmd, "multiply")) {
			pkt->item[pkt->nItem - 1]->scale = atof(data);
		} else if (!stricmp(cmd, "divide")) {
			pkt->item[pkt->nItem - 1]->scale = 1.0 / atof(data);
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"packet\"", cmd);
			error(str);
			return 0;
		}
	}

	return 1;
}

int scriptLoadTelemetry(CFileR *scp)
{
	char line[256], str[256];
	char cmd[256], data[256];

	if (!getScriptLine(scp, line))
	{
		error("Script Error: Unexpected end of file in \"telemetry\"");
		return 0;
	}
	if (line[0] != '{')
	{
		error("Script Error: Expect \"{\" after \"telemetry\"");
		return 0;
	}
	
	while (1)
	{
		if (!getScriptLine(scp, line))
		{
			error("Script Error: Unexpected end of file in \"telemetry\"");
			return 0;
		}

		if (line[0] == '}')
			break;	// found end of block

		SepStr(line, ' ', cmd, data);
		if (!stricmp(cmd, "packetstart"))
		{
			if ((data[0] == '\"') && (data[2] == '\"') && data[1])
			{
				telemetry->chrStart = data[1];
			} else if ((data[0] == '#') && data[1]) {
				telemetry->chrStart = atoi(data+1);
			} else {
				sprintf(str, 
					"Script Error: Packet start/end in \"telemetry\" must be "
					"a single character in quotes or a pound sign followed by an "
					"ASCII code. \"%s\" is invalid", data);
				error(str);
				return 0;
			}
		} else if (!stricmp(cmd, "packetend")) {
			if ((data[0] == '\"') && (data[2] == '\"') && data[1])
			{
				telemetry->chrEnd = data[1];
			} else if ((data[0] == '#') && data[1]) {
				telemetry->chrEnd = atoi(data+1);
			} else {
				sprintf(str, 
					"Script Error: Packet start/end in \"telemetry\" must be "
					"a single character in quotes or a pound sign followed by an "
					"ASCII code. \"%s\" is invalid", data);
				error(str);
				return 0;
			}			
		} else if (!stricmp(cmd, "baud")) {
			telemetry->comBaud = atoi(data);
		} else if (!stricmp(cmd, "port")) {
			strcpy(telemetry->comPort, data);
		} else if (!stricmp(cmd, "packet")) {
			if (!scriptLoadTlmPacket(scp, data))
				return 0;
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in \"telemetry\"", cmd);
			error(str);
			return 0;
		}
	}

	return 1;
}

/*

  main body

*/

int scriptLoad(char *fn)
{
	CFileR scp;
	char line[256], str[256];

	telemetry = new CTelemetry;

	if (!scp.open(fn))
		return 0;

	while (1)
	{
		if (!getScriptLine(&scp, line))
			break;

		if (!stricmp(line, "windows"))
		{
			if (!scriptLoadWindows(&scp))
				return 0;
		} else if (!stricmp(line, "telemetry")) {
			if (!scriptLoadTelemetry(&scp))
				return 0;
		} else {
			sprintf(str, "Script Error: Bad command \"%s\" in main body", line);
			error(str);
			return 0;
		}
	}

	if (telemetry)
		if (!telemetry->initTelemetry())
			return 0;

	scp.close();
	return 1;
}
