/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
sound.cpp

  Warning beeps and so on.
*/

#include <stdio.h>
#include <windows.h>
#include <math.h>
#pragma hdrstop
#define _SOUND_CPP_
#include "sound.h"
#include "uavgs.h"

#ifndef PI
#define PI 3.14159265359
#endif

int soundOK = 0;

int SFreq = 44100;
HWAVEOUT wavo;
WAVEHDR wh;

HGLOBAL wav;
short *wav16;

int initSound()
{
	WAVEFORMATEX wfx;

	// Set things up
	wfx.wFormatTag		= WAVE_FORMAT_PCM;
	wfx.nChannels		= 1;
	wfx.nSamplesPerSec	= SFreq;
	wfx.nAvgBytesPerSec	= SFreq*2;
	wfx.nBlockAlign		= (WORD) (SFreq*2);
	wfx.wBitsPerSample	= 16;
	wfx.cbSize			= 0;
	waveOutOpen(&wavo, WAVE_MAPPER, &wfx,
		0, 0, WAVE_ALLOWSYNC);
	
	wav   = GlobalAlloc(GPTR, SFreq*2);
	wav16 = (short*) wav;

	memset(&wh, 0, sizeof(wh));
	wh.lpData			= (char*) wav;
	wh.dwBufferLength	= SFreq*2;

	waveOutPrepareHeader(wavo, &wh, sizeof(wh));
	
	soundOK = 1;
	return 1;
}

void killSound()
{
	if (!soundOK)
		return;
	soundOK = 0;

	waveOutUnprepareHeader(wavo, &wh, sizeof(wh));
	GlobalFree(wav);
	waveOutReset(wavo);
	waveOutClose(wavo);
}

double SinWav(double t, double Mul)
{
	return sin(2.0 * PI * t * Mul);
}
double SawWav(double t, double Mul)
{
 double f, d;

 f = modf(t * Mul, &d);
 f = (f * 2.0) - 1.0;

 return f;
}

void soundBeepError()
{
	double Mul;
	int i;

	if (!soundOK)
		return;

	// Fill buffer
	// Multiplier = Frequency / Sampling frequency
	memset(wav, 0, SFreq * 2);
	Mul = 600 / (double)SFreq;
	for (i = 0; i < SFreq / 6; i++)
	{
		*(wav16 + i) = (short) (SawWav(i, Mul) * 32767.0);
	}

	Mul = 400 / (double)SFreq;
	for (i = 0; i < SFreq / 4; i++)
	{
		*(wav16 + i) += (short) (SinWav(i, Mul) * 32767.0);
	}

	// Play
	wh.dwBufferLength = SFreq * 2 / 4;
	waveOutWrite(wavo, &wh, sizeof(wh));
}
void soundBeepWarning()
{
	double Mul;
	int i;
	static bool playhalf = 0;

	if ( (playhalf = !playhalf) )
		return;
	
	if (!soundOK)
		return;

	// Fill buffer
	// Multiplier = Frequency / Sampling frequency
	memset(wav, 0, SFreq * 2);
	Mul = 400 / (double)SFreq;
	for (i = 0; i < SFreq / 4; i++)
	{
		*(wav16 + i) = (short) (SinWav(i, Mul) * 32767.0);
	}

	// Play
	wh.dwBufferLength = SFreq * 2 / 4;
	waveOutWrite(wavo, &wh, sizeof(wh));
}

