/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*

  UAV Ground Station

 */

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <richedit.h>
#pragma hdrstop
#define _UAVGS_CPP_
#include "uavgs.h"
#include "packet.h"
#include "map.h"
#include "gauge.h"
#include "miscctl.h"
#include "uavtime.h"
#include "sound.h"

#include "version.h"

HGLOBAL hgMsgBuf = 0;
unsigned msgBufSize = 0;

int showMessageBoxes = 1;
volatile int showLogo = 1;

HBITMAP hbmpBkg, hbmpBkgLogo, hbmpBkgMetal, hbmpBkgGrid;
bool bkgTile;

bool fullscreenLater;

char *strAboutLicense =
	"UAV Ground Station telemetry tool\x0D\x0A"
	"Copyright (C) 2003 Robin Stanton\x0D\x0A\x0D\x0A"
	"Version " UAVGS_VER "\x0D\x0A"
	"uavgs.cpp last compiled " __DATE__ "\x0D\x0A"
	"\x0D\x0A"
	"This program is free software; you can redistribute it and/or modify\x0D\x0A"
	"it under the terms of the GNU General Public License as published by\x0D\x0A"
	"the Free Software Foundation; either version 2 of the License, or\x0D\x0A"
	"(at your option) any later version.\x0D\x0A"
	"\x0D\x0A"
	"This program is distributed in the hope that it will be useful,\x0D\x0A"
	"but WITHOUT ANY WARRANTY; without even the implied warranty of\x0D\x0A"
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\x0D\x0A"
	"GNU General Public License for more details.\x0D\x0A"
	"\x0D\x0A"
	"You should have received a copy of the GNU General Public License\x0D\x0A"
	"along with this program; if not, write to the Free Software\x0D\x0A"
	"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\x0D\x0A";

char *strLogoLicense =
	"\n"
	"UAV Ground Station version " UAVGS_VER ", Copyright (C) 2003 Robin Stanton\n"
	"\n"
	"UAV Ground Station comes with ABSOLUTELY NO WARRANTY.\n"
	"This is free software, and you are welcome to redistribute it"
	"under certain conditions; select `Read License' from the Help menu for details.";

void LoadSettings(int recursing)
{
	HKEY hK;
	LONG l;
	DWORD dw, val;
	char str[256];

 	// Note: If there is a problem (e.g. settings not saved yet) then
	// the defaults will be retained
	strcpy(uavFilename, "default.uav");
	strcpy(mapFilename, "default.map");

	l = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                     "Software\\UAV GroundStation",
	                 0, KEY_ALL_ACCESS, &hK);
	if (l != ERROR_SUCCESS)
	{				
		// Create the key
		if (!recursing)
		{
			SaveSettings();
			LoadSettings(1);
		}
		return;
	}
	
	dw = 4;
	if (RegQueryValueEx(hK, "Background Image", 0, 0, (LPBYTE) &val, &dw) == ERROR_SUCCESS)
		setBkgImage(val);

	dw = 4;
	if (RegQueryValueEx(hK, "GaugeTransparent", 0, 0, (LPBYTE) &val, &dw) == ERROR_SUCCESS)
		gaugeTransparent = val ? 1:0;
	dw = 4;
	if (RegQueryValueEx(hK, "BkgColourKey", 0, 0, (LPBYTE) &val, &dw) == ERROR_SUCCESS)
		bkgColorKey = (int) val;
	dw = 4;
	if (RegQueryValueEx(hK, "Transparent", 0, 0, (LPBYTE) &val, &dw) == ERROR_SUCCESS)
		setTransparent(val ? 1:0);

	dw = 4;
	if (RegQueryValueEx(hK, "Fullscreen", 0, 0, (LPBYTE) &val, &dw) == ERROR_SUCCESS)
		fullscreenLater = val ? 1:0;

	dw = 256;
	if (RegQueryValueEx(hK, "UAVFN", 0, 0, (LPBYTE)str, &dw) == ERROR_SUCCESS)
	{
		strcpy(uavFilename, str);
	}
	dw = 256;
	if (RegQueryValueEx(hK, "MapFN", 0, 0, (LPBYTE)str, &dw) == ERROR_SUCCESS)
	{
		strcpy(mapFilename, str);
	}

	RegCloseKey(hK);
}

void SaveSettings()
{
	HKEY hK;
	LONG l;
	DWORD dw, val;
	char str[256];

    l = RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\UAV GroundStation",
	                   0, "DWORD", 0, KEY_ALL_ACCESS, 0, &hK, &dw);
	if (l!=ERROR_SUCCESS)
		return;
	
	val = bkgImage;
	RegSetValueEx(hK, "Background Image", 0, REG_DWORD, (LPBYTE) &val, 4);

	val = gaugeTransparent;
	RegSetValueEx(hK, "GaugeTransparent", 0, REG_DWORD, (LPBYTE) &val, 4);
	val = bkgColorKey;
	RegSetValueEx(hK, "BkgColourKey", 0, REG_DWORD, (LPBYTE) &val, 4);
	val = isTransparent;
	RegSetValueEx(hK, "Transparent", 0, REG_DWORD, (LPBYTE) &val, 4);

	val = fullscreen;
	RegSetValueEx(hK, "Fullscreen", 0, REG_DWORD, (LPBYTE) &val, 4);

	strcpy(str, uavFilename);
	RegSetValueEx(hK, "UAVFN", 0, REG_SZ, (LPBYTE)str, strlen(str)+1);
	strcpy(str, mapFilename);
	RegSetValueEx(hK, "MapFN", 0, REG_SZ, (LPBYTE)str, strlen(str)+1);

	RegCloseKey(hK);
}

void message(char *msgin)
{
	unsigned newpos, msglen;
	CHARFORMAT cf;
	CHARRANGE cr;
	char *msgBuf;
	char str[256], *msg, msgformat[1024];

	if (!hWndMsg || showMessageBoxes)
	{
		MessageBox(hWnd1, msgin, "UAV Ground Station", MB_ICONINFORMATION | MB_TOPMOST);
	}

	// Decide on formatting
	memset(&cf, 0, sizeof(cf));
	cf.cbSize = sizeof(cf);
	cf.dwMask = CFM_BOLD | CFM_COLOR;
	if (msgin[0] == '!')
	{
		cf.dwEffects |= CFE_BOLD;
		msgin++;
		if (msgin[0] == 'E')			// error
			cf.crTextColor = RGB(128, 0, 0);
		else if (msgin[0] == 'W')		// warning
			cf.crTextColor = RGB(192, 128, 0);
	}
	if ( (!stricmp(msgin, "reset")) || (!stricmp(msgin, "ready")) )
	{
		cf.dwEffects |= CFE_BOLD;
		cf.crTextColor = RGB(0, 128, 0);
	}

	msg = msgformat;
	if (timerGet(str))
		sprintf(msg, "%s\t%s", str, msgin);
	else
		sprintf(msg, "\t%s", msgin);	// timer invalid

	// Create or enlarge message buffer if necessary
	if (!hgMsgBuf)
	{
		msgBufSize = 16;
		hgMsgBuf = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, 16);
	}

	msgBuf = (char*) GlobalLock(hgMsgBuf);

	newpos = strlen(msgBuf);
	msglen = strlen(msg) + 2;

	if (newpos + msglen >= msgBufSize)
	{
		GlobalUnlock(hgMsgBuf);

		msgBufSize = newpos + msglen + 1;
		hgMsgBuf = GlobalReAlloc(hgMsgBuf, msgBufSize, GMEM_ZEROINIT);
		if (!hgMsgBuf)
		{
			sprintf(str, "!!! Message Buffer Error (GlobalReAlloc, %i) !!!", GetLastError());
			SetWindowText(hWndMsg, str);
			return;
		}

		msgBuf = (char*) GlobalLock(hgMsgBuf);
	}

	// Update message buffer
	strcpy(msgBuf + newpos, msg);
	strcat(msgBuf + newpos + msglen - 2, "\x0d\x0a");
	
	// Update message window
	if (hWndMsg)
	{
		SendMessage(hWndMsg, WM_SETREDRAW, 0, 0);	
		
		cr.cpMin = cr.cpMax = newpos;
		SendMessage(hWndMsg, EM_EXSETSEL, 0, (LPARAM) &cr);
		SendMessage(hWndMsg, EM_REPLACESEL, 0, (LPARAM) msgBuf + newpos /* msg and CR LF */);
		
		cr.cpMin = newpos;
		cr.cpMax = newpos + msglen;
		SendMessage(hWndMsg, EM_EXSETSEL, 0, (LPARAM) &cr);
		
		SendMessage(hWndMsg, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM) &cf);
		
		cr.cpMin = cr.cpMax = newpos + msglen;
		SendMessage(hWndMsg, EM_EXSETSEL, 0, (LPARAM) &cr);
		
		// Scroll down to the new message
		SendMessage(hWndMsg, EM_SCROLL, SB_LINEDOWN, 0);
	}

	GlobalUnlock(hgMsgBuf);

	if (hWndMsg)
	{
		SendMessage(hWndMsg, WM_SETREDRAW, 1, 0);
		InvalidateRect(hWndMsg, 0, 1);
		UpdateWindow(hWndMsg);
	}
}
void error(char *msg)
{
	char str[256];

	sprintf(str, "!Error: %s", msg);
	message(str);
}
void fatalError(char *msg)
{
	error(msg);
	MessageBox(hWnd1, msg, "UAV Ground Station", MB_ICONSTOP | MB_TOPMOST);
}

void updateViewMenu()
{
	HMENU hM;
	int id;

	if (!hWndFrame)
		return;

	hM = GetMenu(hWndFrame);

	CheckMenuItem(hM, 4001, MF_BYCOMMAND |
		(isTransparent ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem(hM, 4002, MF_BYCOMMAND |
		(gaugeTransparent ? MF_CHECKED : MF_UNCHECKED));

	// Transparent window background colour menu
	switch (bkgColorKey)
	{
	case BKGC_BLUE:
		id = 4101; 
		break;
	case BKGC_BLACK:
		id = 4102; 
		break;
	case BKGC_GREEN:
		id = 4103; 
		break;
	default:
		error("Bad transparent window colour in UpdateViewMenu()");
		id = 4101;
	}
	CheckMenuRadioItem(hM, 4101, 4103, id, MF_BYCOMMAND);

	// hmm... if fullscreen is checked the menu should be hidden!
	CheckMenuItem(hM, 4011, MF_BYCOMMAND | 
		(fullscreen ? MF_CHECKED : MF_UNCHECKED));

	// Background image
	CheckMenuRadioItem(hM, 4201, 4204, 4200 + bkgImage, MF_BYCOMMAND);
}

void setWindowBkg(HBRUSH hbr)
{
	hbrWndBkg = hbr;
	SetClassLong(hWnd1, GCL_HBRBACKGROUND, (long) hbr);
}
void setGaugeBkg(HBRUSH hbr)
{
	hbrGaugeBkg = hbr;
}
void setTransparent(bool tr)
{
	HBRUSH hbr;

	if (tr)
	{
		switch (bkgColorKey)
		{
		case BKGC_BLUE:
			hbr = hbrBkgBlue;
			break;
		case BKGC_GREEN:
			hbr = hbrBkgGreen;
			break;
		case BKGC_BLACK:
		default:
			hbr = hbrBkgBlack;
		}

		setWindowBkg(hbr);
		if (gaugeTransparent)
			setGaugeBkg(hbr);
		else
			setGaugeBkg(hbrBkgBlack);
	} else {
		setWindowBkg(hbrBkgGrey);

		if (gaugeTransparent)
			// let main window show through
			setGaugeBkg((HBRUSH) GetStockObject(NULL_BRUSH));
		else
			setGaugeBkg(hbrBkgBlack);
	}

	isTransparent = tr;
	updateViewMenu();

	RedrawWindow(hWnd1, 0, 0, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
}
void setFullscreen(bool fs)
{
	int style;

	fullscreen = fs;
	updateViewMenu();

	if (fullscreen)
	{
		// Hide frame window
		ShowWindow(hWndFrame, SW_HIDE);

		// Set main window's parent to desktop
		style = GetWindowLong(hWnd1, GWL_STYLE);
		style &= ! WS_CHILD;
		style |= WS_POPUP;
		SetWindowLong(hWnd1, GWL_STYLE, style);
		SetParent(hWnd1, 0);

		// Maximize main window
		ShowWindow(hWnd1, SW_SHOWMAXIMIZED);
	} else {
		// Set main window's parent to frame window
		SetParent(hWnd1, hWndFrame);
		style = GetWindowLong(hWnd1, GWL_STYLE);
		style &= ! WS_POPUP;
		style |= WS_CHILD;
		SetWindowLong(hWnd1, GWL_STYLE, style);		

		// Show frame window
		ShowWindow(hWndFrame, SW_SHOW);

		// Restore main window
		ShowWindow(hWnd1, SW_SHOWNORMAL);

		RedrawWindow(GetDesktopWindow(), 0, 0, RDW_INVALIDATE | RDW_ALLCHILDREN | RDW_ERASENOW);
	}
}

void updateBkgBmp(HWND handle, int szX = 0, int szY = 0)
{
	HDC hDC, hMem, hMem2;
	RECT rc;
	BITMAP bm;
	int x,y;

	if (!szX && !szY)
		GetClientRect(handle, &rc);
	else {
		rc.left = rc.top = 0;
		rc.right  = szX;
		rc.bottom = szY;
	}

	// Stretch background bitmap to proper size
	if (hbmpWndBkg)
		DeleteObject(hbmpWndBkg);
	hDC = GetDC(handle);
	hMem = CreateCompatibleDC(hDC);
	hbmpWndBkg = CreateCompatibleBitmap(hDC, rc.right, rc.bottom);
	SelectObject(hMem, hbmpWndBkg);
	
	if (hbmpBkg)
	{
		GetObject(hbmpBkg, sizeof(bm), &bm);
		
		hMem2 = CreateCompatibleDC(hDC);
		
		SelectObject(hMem2, hbmpBkg);

		if (bkgTile)
		{
			// Tile bitmap
			for (x = 0; x < rc.right; x += bm.bmWidth)
				for (y = 0; y < rc.bottom; y += bm.bmHeight)
				{
					BitBlt(hMem, x,y, bm.bmWidth, bm.bmHeight, 
						hMem2, 0,0, SRCCOPY);
				}
		} else {
			// Stretch to fit
			StretchBlt(hMem, 0,0, rc.right, rc.bottom, 
				hMem2, 0,0, bm.bmWidth, bm.bmHeight, SRCCOPY);
		}
	
		DeleteDC(hMem2);
	} else {
		// normal window background
		FillRect(hMem, &rc, (HBRUSH) GetStockObject(LTGRAY_BRUSH));
	}
	
	DeleteDC(hMem);
	ReleaseDC(handle, hDC);
}
void setBkgImage(int img)
{
	bkgImage = img;

	switch (img)
	{
	case BKG_NONE:
		hbmpBkg = 0;
		break;
	case BKG_LOGO:
		hbmpBkg = hbmpBkgLogo;
		bkgTile = 0;
		break;
	case BKG_METAL:
		hbmpBkg = hbmpBkgMetal;
		bkgTile = 1;
		break;
	case BKG_GRID:
		hbmpBkg = hbmpBkgGrid;
		bkgTile = 1;
		break;
	}

	updateViewMenu();

	updateBkgBmp(hWnd1);
	RedrawWindow(hWnd1, 0, 0, RDW_INVALIDATE | RDW_ALLCHILDREN | RDW_ERASENOW);
}

// Select a UAV script and open it
void openUAV()
{
	char fn[1024];
	OPENFILENAME ofn;
	char ft[1024];
	RECT rc;

	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize		= sizeof(ofn);
	ofn.hwndOwner		= hWnd1;
    ofn.lpstrFilter		= 
		"UAV Scripts (*.uav)\0*.uav\0"
		"All Files (*.*)\0*.*\0";
	ofn.lpstrFile		= fn; strcpy(fn,"");
	ofn.nMaxFile		= 1023;
	ofn.lpstrFileTitle	= ft;
	ofn.nMaxFileTitle	= 1023;
	ofn.lpstrTitle		= "Open UAV";
	ofn.Flags			= OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
    ofn.lpstrDefExt		= "uav";
	
	if (!GetOpenFileName(&ofn))
		return;

	strcpy(uavFilename, fn);

	// Clear old stuff
	if (telemetry)
	{
		delete telemetry;
		telemetry = 0;
	}
	hWndMsg = 0;
	wndLayout.frames.destroy();
	wndLayout.frames.init();
	deleteVars();

	// Load new script
	if (!scriptLoad(uavFilename))
	{
		fatalError("Failed to load configuration script");
		return;
	}

	GetClientRect(hWnd1, &rc);
	wndLayout.arrange(rc.right, rc.bottom);
}

// Select a map script and open it
void openMap()
{
	char fn[1024];
	OPENFILENAME ofn;
	char ft[1024];

	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize		= sizeof(ofn);
	ofn.hwndOwner		= hWnd1;
    ofn.lpstrFilter		= 
		"Map Scripts (*.map)\0*.map\0"
		"All Files (*.*)\0*.*\0";
	ofn.lpstrFile		= fn; strcpy(fn,"");
	ofn.nMaxFile		= 1023;
	ofn.lpstrFileTitle	= ft;
	ofn.nMaxFileTitle	= 1023;
	ofn.lpstrTitle		= "Open Map";
	ofn.Flags			= OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
    ofn.lpstrDefExt		= "map";
	
	if (!GetOpenFileName(&ofn))
		return;

	strcpy(mapFilename, fn);

	MessageBox(hWnd1,
		"The new map will be loaded the next time the program starts or\n"
		"a UAV script is loaded",
		"UAV GroundStation", MB_ICONINFORMATION);
}

LRESULT CALLBACK dpAbout(HWND handle, UINT msg,
						 WPARAM wParam, LPARAM lParam)
{
	char str[256], str2[256];

	switch (msg)
	{
	case WM_CLOSE:
		EndDialog(handle, 123);
		break;
	case WM_COMMAND:
		switch (wParam)
		{
		case IDOK:
		case IDCANCEL:
			EndDialog(handle, 123);
			break;
		}
		break;
	case WM_INITDIALOG:
		GetDlgItemText(handle, 101, str, 255);
		sprintf(str2, str, UAVGS_VER, __DATE__);
		SetDlgItemText(handle, 101, str2);

		SetDlgItemText(handle, 102, strAboutLicense);
		return(1);
	default:
		return(0);
	}
	return(1);
}
LRESULT CALLBACK dpLogo(HWND handle, UINT msg,
						WPARAM wParam, LPARAM lParam)
{
	char str[512];
	/*
	sum of 2 numbers = minimum time logo shows
	2nd number = how long it stays after the program loads
	*/
	static int logoTimeLeft = 5;
	static int logoTimeLeft2 = 10;

	switch (msg)
	{
	case WM_CLOSE:
		break;
	case WM_COMMAND:
		switch (wParam)
		{
		case IDOK:
		case IDCANCEL:
			break;
		}
		break;
	case WM_INITDIALOG:
		SetWindowPos(handle, HWND_TOPMOST, 0,0, 0,0, 
			SWP_NOMOVE | SWP_NOSIZE);

		sprintf(str, strLogoLicense, UAVGS_VER);
		SetDlgItemText(handle, 101, str);

		SetTimer(handle, 1, 100, 0);
		return(1);
	case WM_TIMER:
		if (logoTimeLeft)
			logoTimeLeft--;
		else if (!showLogo)
		{
			if (logoTimeLeft2)
				logoTimeLeft2--;
			else {
				SetForegroundWindow(hWnd1);
				EndDialog(handle, 123);
			}
		}
		break;
	default:
		return(0);
	}
	return(1);
}

LRESULT CALLBACK wpMain(HWND handle, UINT msg,
						WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	char str[256];
	HDC hDC, hMem;
	RECT rc;
	
	switch (msg)
	{
	case WM_CLOSE:
		if (fullscreen)
			// pass it on
			PostMessage(hWndFrame, WM_CLOSE, 0, 0);
		break;
	
	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_ESCAPE:
			if (fullscreen)
				setFullscreen(0);
			break;
		}
		break;
    case WM_DESTROY:
		KillTimer(handle, 1);
		KillTimer(handle, 2);
		break;
		
	case WM_SIZE:
		updateBkgBmp(handle, LOWORD(lParam), HIWORD(lParam));
		wndLayout.arrange(LOWORD(lParam), HIWORD(lParam));
		break;
	
	case WM_CREATE:
		// Timer to update mission clock
		SetTimer(handle, 1, 500, 0);

		// Flash timer
		SetTimer(handle, 2, 200, 0);

		// Warning beep timer
		SetTimer(handle, 3, 300, 0);
		break;

	case WM_PAINT:
		BeginPaint(handle, &ps);
		EndPaint(handle, &ps);
		break;

	case WM_TIMER:
		switch (wParam)
		{
		case 1:	// Update mission time
			timerGet(str);
			SendMessage(hWndStatus, SB_SETTEXT, 3, (LPARAM) str);
			break;
		case 2: // Flash gauges
			gaugeFlash = !gaugeFlash;
			RedrawWindow(hWnd1, 0, 0, RDW_ERASE | RDW_INVALIDATE | RDW_ALLCHILDREN);
			RedrawWindow(hWnd1, 0, 0, RDW_VALIDATE | RDW_NOCHILDREN);
			break;
		case 3: // Warning beep
			if (beepProblem & BEEP_ERROR)
				soundBeepError();
			else if (beepProblem & BEEP_WARN)
				soundBeepWarning();

			// update
			beepProblem &= ! (BEEP_ERROR | BEEP_WARN);
			RedrawWindow(hWnd1, 0, 0, RDW_ERASE | RDW_INVALIDATE | RDW_ALLCHILDREN);
			RedrawWindow(hWnd1, 0, 0, RDW_VALIDATE | RDW_NOCHILDREN);
			break;
		}
		break;

	case WM_ERASEBKGND:
		if (!hbmpWndBkg || isTransparent)
			return(DefWindowProc(handle,msg,wParam,lParam));

		GetClientRect(handle, &rc);

		hDC = (HDC) wParam;
		hMem = CreateCompatibleDC(hDC);
		SelectObject(hMem, hbmpWndBkg);
		BitBlt(hDC, 0,0, rc.right, rc.bottom, hMem, 0,0, SRCCOPY);
		DeleteDC(hMem);
		break;
	default:
		return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}

LRESULT CALLBACK wpFrame(HWND handle, UINT msg,
						 WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	RECT r, r2;
	INT parts[4];

	switch (msg)
	{
	case WM_CLOSE:
    case WM_DESTROY:
		PostQuitMessage(0);
		break;
		
	case WM_SIZE:
		// forward message to status bar
		SendMessage(hWndStatus, WM_SIZE, wParam, lParam);

		// rearrange status bar items
		parts[0] = max(LOWORD(lParam) - 300, 20);
		parts[1] = parts[0] + 100;
		parts[2] = parts[1] + 100;
		parts[3] = -1;
		SendMessage(hWndStatus, SB_SETPARTS, 4, (LPARAM) parts);

		// resize the main window so it fills the remaining space
		r.top = 0;
		r.bottom = HIWORD(lParam);
		
		GetWindowRect(hWndStatus, &r2);
		r.bottom -= (r2.bottom - r2.top);
		
		MoveWindow(hWnd1, 0, r.top, LOWORD(lParam), r.bottom, 0);
		break;
		
	case WM_CREATE:
		break;
		
	case WM_PAINT:
		BeginPaint(handle, &ps);
		EndPaint(handle, &ps);
		break;

	case WM_COMMAND:
		switch (wParam)
		{
		// File
		case 1001:	// Open UAV
			openUAV();
			break;
		case 1002:	// Open Map
			openMap();
			break;
		case 1099:	// Exit
			PostQuitMessage(0);
			break;
	
		// UAV
		case 2001:	// Reset Timer
			timerReset();
			break;

		// Maps

		// View
		case 4001:	// Transparent Window
			setTransparent(!isTransparent);
			break;
		case 4101:	// Transparent Colour > Blue
			bkgColorKey = BKGC_BLUE;
			setTransparent(isTransparent);
			break;
		case 4102:	// Transparent Colour > Black
			bkgColorKey = BKGC_BLACK;
			setTransparent(isTransparent);
			break;
		case 4103:	// Transparent Colour > Green
			bkgColorKey = BKGC_GREEN;
			setTransparent(isTransparent);
			break;
		case 4002:	// Transparent Gauges
			gaugeTransparent = !gaugeTransparent;
			setTransparent(isTransparent);
			break;
		case 4011:	// Full Screen
			setFullscreen(!fullscreen);
			break;
		case 4201:	// Background Image > *
		case 4202:
		case 4203:
		case 4204:
			setBkgImage(LOWORD(wParam) - 4200);
			break;

		// Help
		case 9098:	// Read License
			ShellExecute(handle, 0, "copying.txt", 0, 0, SW_SHOW);
			break;
		case 9099:	// About
			DialogBox(hInstance, "ABOUT", handle, (DLGPROC) dpAbout);
			break;
		}
		break;
		
	default:
		return(DefWindowProc(handle,msg,wParam,lParam));
	}
	return 0L;
}

DWORD WINAPI threadLogo(LPVOID lpvParam)
{
	DialogBox(hInstance, "LOGO", GetDesktopWindow(), (DLGPROC) dpLogo);
	return 0;
}

int init()
{
	WNDCLASS wc;
	LOGFONT lf;
	int h;
	HDC hDC;
	DWORD dw;
	
	hWndMsg = hWndFrame = hWnd1 = 0;
	hbmpWndBkg = hbmpBkg = 0;

	beepProblem = 0;
	gaugeFlash = 0;

	// Show logo screen
	CreateThread(0, 0, threadLogo, 0, 0, &dw);

	// Create fonts
	hDC = GetDC(GetDesktopWindow());
	h = MulDiv(10 /*points*/, GetDeviceCaps(hDC, LOGPIXELSY), 72);
	ReleaseDC(GetDesktopWindow(), hDC);
	
	memset(&lf, 0, sizeof(lf));
	lf.lfHeight = -h;
	lf.lfOutPrecision = OUT_TT_PRECIS;
	strcpy(lf.lfFaceName, "Courier New");
	hfDefault = CreateFontIndirect(&lf);

	// Create brushes
	hbrBkgGrey	= (HBRUSH) GetStockObject(LTGRAY_BRUSH);
	hbrBkgBlack	= (HBRUSH) GetStockObject(BLACK_BRUSH);
	hbrBkgBlue	= CreateSolidBrush(RGB(0, 0, 255));
	hbrBkgGreen	= CreateSolidBrush(RGB(0, 255, 0));

	gaugeTransparent = 0;
	bkgColorKey = BKGC_BLUE;
	setTransparent(0);
	setFullscreen(0);
	
	setBkgImage(BKG_NONE);	// Temporary

	// Create frame window
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpFrame;
	wc.hInstance	 = hInstance;
	wc.hIcon		 = LoadIcon(hInstance, MAKEINTRESOURCE(1));
	wc.hCursor		 = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
	wc.lpszClassName = "UAVGSfr";
	wc.lpszMenuName	 = "MENU_1";
	if (!RegisterClass(&wc))
		return 0;

	hWndFrame = CreateWindowEx(0, "UAVGSfr", STR_WNDTITLE,
		WS_OVERLAPPEDWINDOW, 0,0, 512,384,
		GetDesktopWindow(), 0, hInstance, 0);
	if (!hWndFrame) return 0;

	updateViewMenu();

	hWndStatus = CreateStatusWindow(SBARS_SIZEGRIP | WS_CHILD | WS_VISIBLE, 
		"UAV Ground Station", hWndFrame, 999);
	if (!hWndStatus) return 0;

	// Create main window
	memset(&wc, 0, sizeof(wc));
	wc.lpfnWndProc	 = (WNDPROC) wpMain;
	wc.hInstance	 = hInstance;
	wc.hIcon		 = LoadIcon(hInstance, MAKEINTRESOURCE(1));
	wc.hCursor		 = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = hbrWndBkg;
	wc.lpszClassName = "UAVGS";
	if (!RegisterClass(&wc))
		return 0;

	hWnd1 = CreateWindowEx(0, "UAVGS", "",
		WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN, 0,0, 512,384,
		hWndFrame, 0, hInstance, 0);
	if (!hWnd1) return 0;

	// Create other window classes
	if (!regClassesMap())
		return 0;
	if (!regClassesGauge())
		return 0;
	if (!regClassesMisc())
		return 0;
	
	// Load bitmaps
	hbmpBkgLogo = (HBITMAP) LoadImage(0, "background logo.bmp", IMAGE_BITMAP, 0,0, LR_LOADFROMFILE);
	if (!hbmpBkgLogo)
		message("!Warning: A background bitmap did not load");
	hbmpBkgMetal = (HBITMAP) LoadImage(0, "background metal.bmp", IMAGE_BITMAP, 0,0, LR_LOADFROMFILE);
	if (!hbmpBkgMetal)
		message("!Warning: A background bitmap did not load");
	hbmpBkgGrid = (HBITMAP) LoadImage(0, "background grid.bmp", IMAGE_BITMAP, 0,0, LR_LOADFROMFILE);
	if (!hbmpBkgGrid)
		message("!Warning: A background bitmap did not load");

	setBkgImage(BKG_LOGO);

	LoadSettings();

	// Load script
	if (!scriptLoad(uavFilename))
	{
		fatalError("Failed to load configuration script");
		return 0;
	}

	showMessageBoxes = 0;
	// the main window will be visible now, so if there is a message window,
	// use it.

	if (hWndMsg)
		message("!--- UAV Ground Station ---");
	timerReset();

	if (!initSound())
		message("!Warning: Error initializing sound");

	InvalidateRect(hWnd1, 0, 0);
	UpdateWindow(hWnd1);

	return 1;
}

int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nShowCmd)
{
	MSG msg;

	hInstance = hInst;

	InitCommonControls();
	LoadLibrary("Riched32.dll");

	if (!init())
		return 0;

	// FIXME: Does not respect nShowCmd
	// Note that maximized should be the default, not restored
	ShowWindow(hWndFrame, SW_SHOWMAXIMIZED);
	UpdateWindow(hWndFrame);

	// fullscreenLater is specified in LoadSettings()
	// (which runs before the window is ready)
	if (fullscreenLater)
		setFullscreen(1);

	showLogo = 0;

	/* Handle messages */
	while (GetMessage(&msg, (HWND) NULL, 0, 0)) 
	{ 
		TranslateMessage(&msg); 
		DispatchMessage(&msg); 
	} 

	SaveSettings();
	
	if (telemetry)
		telemetry->stopTelemetry();
	deleteVars();
	if (hgMsgBuf)
		GlobalFree(hgMsgBuf);

	killSound();

	return 1;
}
