/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
wndlayout.cpp

  Window layout.
*/

#include <stdio.h>
#include <windows.h>
#pragma hdrstop
#define _WNDLAYOUT_CPP_
#include "wndlayout.h"
#include "uavgs.h"

int curControlID = 1001;

/*
 
   CWndLayout

*/
void CWndLayout::arrange(int x, int y)
{
	frames.home();
	while (!frames.pastend())
	{
		frames.data()->arrange(0, 0, x, y);
		++frames;
	}

	// Repaint main window
	InvalidateRect(hWnd1, 0, 1);
	UpdateWindow(hWnd1);
}
CWndLayout::CWndLayout()
{
	frames.init();
}
CWndLayout::~CWndLayout()
{
	frames.destroy();
}

/*

   CWndFrame

*/
CWndFrame::CWndFrame()
{
	x = y = 0.0;
	w = h = 1.0;
	padding = 0;
	frames.init();
	objects.init();
}
CWndFrame::~CWndFrame()
{		
	frames.destroy();
	objects.destroy();
}
/*
 Position the frame inside its parent
 
 (call with parent's size and position)
*/
void CWndFrame::arrange(int frX, int frY, int frW, int frH)
{
	int myX, myY, myW, myH;

	myX = frX + (int) (frW * x) + padding;
	myY = frY + (int) (frH * y) + padding;
	myW = (int) (frW * w) - padding * 2;
	myH = (int) (frH * h) - padding * 2;

	frames.home();
	while (!frames.pastend())
	{
		frames.data()->arrange(myX, myY, myW, myH);
		++frames;
	}

	objects.home();
	while (!objects.pastend())
	{
		objects.data()->arrange(myX, myY, myW, myH);
		++objects;
	}
}

/*

   CWndFrameObj

*/
CWndFrameObj::CWndFrameObj()
{
	x = y = 0.0;
	w = h = 1.0;
	padding = 0;
	
	name[0] = 0;
	
	handle = 0;
	
	type = 0;
}
CWndFrameObj::~CWndFrameObj()
{
	switch (type)
	{
	case WNDOT_MAP:
		if (map.map) delete map.map;
		break;
	}

	if (handle)
		DestroyWindow(handle);
}

/*
Position the window inside its parent frame
*/
void CWndFrameObj::arrange(int frX, int frY, int frW, int frH)
{
	int myX, myY, myW, myH;

	myX = frX + (int) (frW * x) + padding;
	myY = frY + (int) (frH * y) + padding;
	myW = (int) (frW * w) - padding * 2;
	myH = (int) (frH * h) - padding * 2;

	if (handle)
	{
		MoveWindow(handle, myX, myY, myW, myH, 1);
	}
}

/*
 Create a text box object
*/
void CWndFrameObj::createTextbox(int multiLine)
{
	if (multiLine)
		handle = CreateWindowEx(WS_EX_CLIENTEDGE, "RichEdit", "",
			WS_CHILD | WS_VISIBLE | WS_HSCROLL |
			ES_MULTILINE | WS_VSCROLL/* | ES_DISABLENOSCROLL*/
			| ES_READONLY,
			0,0, 16,16, hWnd1, (HMENU) curControlID++, hInstance, 0);
	else
		handle = CreateWindowEx(WS_EX_CLIENTEDGE, "STATIC", "",
			WS_CHILD | WS_VISIBLE |	SS_CENTER | SS_NOPREFIX,
			0,0, 16,16, hWnd1, (HMENU) curControlID++, hInstance, 0);
	
	if (!handle) 
	{
		error("Could not create text box (RichEdit)");
		return;
	}
	
	SendMessage(handle, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), 1);

	type = WNDOT_TEXTBOX;
}

/*
 Create an altimeter object
*/
void CWndFrameObj::createAltimeter()
{
	handle = CreateWindowEx(0, "UAVGSalt", "",
		WS_CHILD | WS_VISIBLE,
		0,0, 16,16, hWnd1, 0, hInstance, 0);	

	if (!handle)
	{
		error("Could not create altimeter");
		return;
	}

	SetProp(handle, "VAR", 0);
	type = WNDOT_ALTIMETER;
}

/*
 Create an compass object
*/
void CWndFrameObj::createCompass()
{
	handle = CreateWindowEx(0, "UAVGScompass", "",
		WS_CHILD | WS_VISIBLE,
		0,0, 16,16, hWnd1, 0, hInstance, 0);	

	if (!handle)
	{
		error("Could not create compass");
		return;
	}

	SetProp(handle, "VAR", 0);
	type = WNDOT_COMPASS;
}

/*
 Create a round gauge object
*/
void CWndFrameObj::createRoundGauge()
{
	handle = CreateWindowEx(0, "UAVGSroundg", "",
		WS_CHILD | WS_VISIBLE,
		0,0, 16,16, hWnd1, 0, hInstance, 0);	

	if (!handle)
	{
		error("Could not create round gauge");
		return;
	}

	SetProp(handle, "VAR", 0);
	SetProp(handle, "WFO", this);
	type = WNDOT_ROUND_GAUGE;
	memset(&roundGauge, 0, sizeof(roundGauge));
}

/*
 Create an LCD object
*/
void CWndFrameObj::createLCD()
{
	handle = CreateWindowEx(0, "UAVGSlcd", "",
		WS_CHILD | WS_VISIBLE,
		0,0, 16,16, hWnd1, 0, hInstance, 0);	

	if (!handle)
	{
		error("Could not create LCD");
		return;
	}

	SetProp(handle, "VAR", 0);
	SetProp(handle, "WFO", this);
	type = WNDOT_LCD;
	memset(&lcd, 0, sizeof(lcd));
}

/*
 Create an LCD Text object
*/
void CWndFrameObj::createLCDText()
{
	handle = CreateWindowEx(0, "UAVGSlcdtxt", "",
		WS_CHILD | WS_VISIBLE,
		0,0, 16,16, hWnd1, 0, hInstance, 0);	

	if (!handle)
	{
		error("Could not create LCD Text");
		return;
	}

	SetProp(handle, "VAR", 0);
	SetProp(handle, "WFO", this);
	type = WNDOT_LCDTEXT;
	memset(&lcdtext, 0, sizeof(lcdtext));
}

/*
 Create a map object
*/
void CWndFrameObj::createMap()
{
	handle = CreateWindowEx(0, "UAVGSmap", "",
		WS_CHILD | WS_VISIBLE,
		0,0, 16,16, hWnd1, 0, hInstance, 0);	

	if (!handle)
	{
		error("Could not create map");
		return;
	}

	SetProp(handle, "WFO", this);
	type = WNDOT_MAP;
	memset(&map, 0, sizeof(map));
}

/*
 Create a label object
*/
void CWndFrameObj::createLabel()
{
	handle = CreateWindowEx(0, "UAVGSlabel", "",
		WS_CHILD | WS_VISIBLE,
		0,0, 16,16, hWnd1, 0, hInstance, 0);	

	if (!handle)
	{
		error("Could not create label");
		return;
	}

	SetProp(handle, "WFO", this);
	type = WNDOT_LABEL;
}

void CWndFrameObj::update(CVariable *var)
{
	switch (type)
	{
	case WNDOT_TEXTBOX:
		SetWindowText(handle, var->getString());
		break;
	case WNDOT_ALTIMETER:
	case WNDOT_COMPASS:
	case WNDOT_ROUND_GAUGE:
	case WNDOT_LCD:
	case WNDOT_LCDTEXT:
		SetProp(handle, "VAR", (HANDLE) var);
		InvalidateRect(handle, 0, 0);
		UpdateWindow(handle);
		break;
	case WNDOT_LABEL:
		/*
		Note:
		 The label name may happen to match the variable
		shown in whatever it's labelling. Don't whine.
		*/
		InvalidateRect(handle, 0, 0);
		UpdateWindow(handle);
		break;
	default:
		error("A window that does not show data has been assigned to a variable (a map perhaps?)");
		break;
	}
}
