/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
wndlayout.h

  Window layout.
*/

#ifndef _WNDLAYOUT_H_
#define _WNDLAYOUT_H_

#ifndef _WNDLAYOUT_CPP_
#define _WNDLAYOUT_CPP_ extern
#endif

class CWndFrameObj;
class CWndFrame;
class CWndLayout;

#include "linkedlist.h"
#include "packet.h"
#include "map.h"

// Window inside a frame
#define WNDOT_TEXTBOX		1
#define WNDOT_MAP			2
#define WNDOT_MINIMAP		3
#define WNDOT_ALTIMETER		4
#define WNDOT_COMPASS		5
#define WNDOT_ROUND_GAUGE	6
#define WNDOT_LCD			7
#define WNDOT_LCDTEXT		8
#define WNDOT_LABEL			9

struct wfodRoundGauge {
	double minAngle, maxAngle;
	double minValue, maxValue;
	
	bool doTicks;
	double tickSpace;
	bool doTicks2;
	double tickSpace2;
	
	bool doWarnLo, doWarnHi;
	double warnLo, warnHi;
	bool doErrorLo, doErrorHi;
	double errorLo, errorHi;
};
struct wfodLCD {
	int nDigits;
	double msDigit;
};
struct wfodLCDText {
	int size;
};
struct wfodMap {
	CMap *map;
};
struct wfodLabel {
	double size;
	char text[256];
};

class CWndFrameObj
{
public:	
	double x,y, w,h;	// Size and position (fractions of parent's dimensions)
	int padding;		// Pixels of blank space to leave as a border
	char name[64];
	HWND handle;

	int type;			// WNDOT_*
	union {
		wfodRoundGauge roundGauge;
		wfodLCD lcd;
		wfodLCDText lcdtext;
		wfodMap map;
		wfodLabel label;
	};

	CWndFrameObj();
	~CWndFrameObj();

	void arrange(int frX, int frY, int frW, int frH);
	void createTextbox(int multiLine);
	void createAltimeter();
	void createCompass();
	void createRoundGauge();
	void createLCD();
	void createLCDText();
	void createMap();
	void createLabel();
	void update(CVariable *var);
};

class CWndFrame
{
public:
	CLList<CWndFrame*> frames;
	CLList<CWndFrameObj*> objects;

	double x,y, w,h;	// Size and position (fractions of parent's dimensions)
	int padding;		// Pixels of blank space to leave as a border

	CWndFrame();
	~CWndFrame();

	void arrange(int frX, int frY, int frW, int frH);
};

class CWndLayout
{
public:
	CLList<CWndFrame*> frames;

	CWndLayout();
	~CWndLayout();

	void arrange(int x, int y);
};

#endif
